from __future__ import print_function
from test_orm import OrloDbTest
from test_base import ReleaseDbUtil
from flask_testing import LiveServerTestCase
from orlo.orm import db
import orlo
import requests
import os

__author__ = 'alforbes'

try:
    TRAVIS = True if os.environ['TRAVIS'] == 'true' else False
except KeyError:
    TRAVIS = False


class StressTest(LiveServerTestCase, ReleaseDbUtil):
    """ This test was to track down a database connection leak

    It stands because it detected the leak by stalling on db.drop_all() in
    the tearDown function
    """
    ITERATIONS = 1

    def create_app(self):
        if TRAVIS:
            db_uri = 'postgres://orlo:password@localhost:5432/orlo'
        else:
            db_uri = 'postgres://orlo:password@192.168.57.100:5432/orlo'

        app = orlo.app
        app.config['SQLALCHEMY_DATABASE_URI'] = db_uri
        # app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite://'
        app.config['TESTING'] = True
        app.config['DEBUG'] = False
        app.config['TRAP_HTTP_EXCEPTIONS'] = True
        app.config['PRESERVE_CONTEXT_ON_EXCEPTION'] = False
        app.config['LIVESERVER_PORT'] = 8943
        app.config['LIVESERVER_TIMEOUT'] = 5

        return orlo.app

    def setUp(self):
        db.get_engine(self.app).dispose()
        db.create_all()
        db.session.begin_nested()

    def tearDown(self):
        db.session.rollback()
        db.session.close()
        db.drop_all()  # stall here implies connection leak
        db.session.remove()
        db.get_engine(self.app).dispose()

    def _get_releases(self, release_id=None, filters=None, expected_status=200):
        """
        Perform a GET to /releases with optional filters
        """

        if release_id:
            path = '{}/releases/{}'.format(self.get_server_url(),
                                           release_id)
        elif filters:
            path = '{}/releases?{}'.format(self.get_server_url(),
                                           '&'.join(filters))
        else:
            path = '{}/releases'.format(self.get_server_url())

        results_response = requests.get(
            path, headers={'content_type': 'application/json'}
        )

        self.assertEqual(results_response.status_code, expected_status)
        return results_response.json

    def test_hammer_postgres(self):
        """ Hammer postgres and see if it times out """
        for i in range(0, self.ITERATIONS):
            if i+1 % 50 == 0:
                print('Creating release {}'.format(i+1))
            OrloDbTest._create_finished_release(success=True)

        for i in range(0, self.ITERATIONS):
            print('get', i)
            self._get_releases(filters=['user=testuser'])




from __future__ import print_function
from datetime import date, timedelta, datetime

from orlo.util import append_or_create_platforms

from test_orm import OrloDbTest
from orlo.orm import db, Release, Package

__author__ = 'alforbes'

"""
This tests the stats functions by creating a known dataset,
making the results predictable
"""


def date_range(start, end):
    # Courtesy of http://stackoverflow.com/questions/1060279
    for n in range(int((end - start).days)):
        yield start + timedelta(n)


class TestStatsEmpirical(OrloDbTest):
    """
    Test release stats by creating a known data set

    Changing the dates or number of releases may impact tests
    """

    # Create releases for two days which span a year
    start_date = datetime(2015, 12, 31)
    end_date = datetime(2016, 1, 2)

    # Which means results for 2015-12-31 and 2016-01-01 should match below:
    normal_successful_per_day = 2
    normal_failed_per_day = 1
    rollback_successful_per_day = 1
    rollback_failed_per_day = 1

    package_list = ['p1', 'p2']

    time_cursor = start_date

    def setUp(self):
        self.releases = 0
        super(OrloDbTest, self).setUp()

        for day in date_range(self.start_date, self.end_date):
            if day.weekday() > 5:
                # weekend! don't release
                continue

            # starting at 9am...
            self.time_cursor = day + timedelta(hours=9)

            # do $per_day releases...
            for i in range(0, self.normal_successful_per_day):
                self.create_release(True, True)
            for i in range(0, self.normal_failed_per_day):
                self.create_release(True, False)
            for i in range(0, self.rollback_successful_per_day):
                self.create_release(False, True)
            for i in range(0, self.rollback_failed_per_day):
                self.create_release(False, False)

            db.session.commit()
        # print("Total releases created: {}".format(self.releases))

    def create_release(self, normal, successful, user='test_user', team='test_team',
                       platform='test_platform'):
        release = Release(
                platforms=append_or_create_platforms([platform]),
                user=user,
                team=team,
                # references=list_to_string(['test_reference'])
        )
        release.stime = self.time_cursor
        db.session.add(release)

        for package in self.package_list:
            package = Package(
                release_id=release.id,
                name=package,
                version='0.0.0'
            )
            # which take 10 mins each...
            package.stime = self.time_cursor
            package.ftime = self.time_cursor = self.time_cursor + timedelta(minutes=10)
            if normal:
                package.rollback = False
            else:
                package.rollback = True
            if successful:
                package.status = 'SUCCESSFUL'
            else:
                package.status = 'FAILED'
            db.session.add(package)

        release.ftime = self.time_cursor
        release.duration = release.ftime - release.stime
        self.releases += 1

    def test_stats(self):
        """
        Test our data against /stats
        """
        response = self.client.get('/stats').json
        total = response['global']['releases']['total']
        self.assertEqual(total['successful'] + total['failed'],  self.releases)

    def test_stats_user(self):
        """
        Test /stats/user with data
        """
        # Add a release with a different user
        self.create_release(False, False, user='bad_user')

        response = self.client.get('/stats/user/test_user').json
        total = response['test_user']['releases']['total']
        self.assertEqual(total['successful'] + total['failed'], self.releases - 1)

    def test_stats_team(self):
        """
        Test /stats/team with data
        """
        # Add a release with a different team
        self.create_release(False, False, team='bad_team')

        response = self.client.get('/stats/team/test_team').json
        total = response['test_team']['releases']['total']
        self.assertEqual(total['successful'] + total['failed'], self.releases - 1)

    def test_stats_platform(self):
        """
        Test /stats/platform with data
        """
        # Add a release with a different team
        self.create_release(False, False, platform='bad_platform')

        response = self.client.get('/stats/platform/test_platform').json
        total = response['test_platform']['releases']['total']
        self.assertEqual(total['successful'] + total['failed'], self.releases - 1)

    def test_stats_package(self):
        """
        Test /stats/package with data
        """
        # All packages are in every release here, could be better tested
        response = self.client.get('/stats/package/p1').json
        total = response['p1']['releases']['total']
        self.assertEqual(total['successful'] + total['failed'], self.releases)

    def test_stats_by_date_release(self):
        """
        Tests /stats/by_date/release
        """
        s_year = self.start_date.year
        s_month = self.start_date.month
        response = self.client.get('/stats/by_date/release')

        self.assertEqual(
                response.json[str(s_year)][str(s_month)]['normal']['successful'],
                self.normal_successful_per_day,
        )
        self.assertEqual(
                response.json[str(s_year)][str(s_month)]['normal']['successful'],
                self.normal_successful_per_day,
        )
        self.assertEqual(
                response.json[str(s_year + 1)]['1']['rollback']['failed'],
                self.rollback_failed_per_day
        )

    def test_stats_by_date_package(self):
        """
        Tests /stats/by_date/package
        """
        s_year = self.start_date.year
        s_month = self.start_date.month
        response = self.client.get('/stats/by_date/package')

        # Assumes p1 is in every release
        self.assertEqual(
            response.json[str(s_year)][str(s_month)]['p1']['normal']['successful'],
            self.normal_successful_per_day,
        )
        self.assertEqual(
            response.json[str(s_year)][str(s_month)]['p2']['normal']['successful'],
            self.normal_successful_per_day,
        )
        self.assertEqual(
            response.json[str(s_year + 1)]['1']['p1']['rollback']['failed'],
            self.rollback_failed_per_day
        )

    def test_stats_by_date_release_with_unit_day(self):
        """
        Tests /stats/by_date/release
        """
        s_year = self.start_date.year
        s_month = self.start_date.month
        response = self.client.get('/stats/by_date/release?unit=day')

        self.assertEqual(
                response.json[str(s_year)][str(s_month)]['31']['normal']['successful'],
                self.normal_successful_per_day,
        )
        self.assertEqual(
                response.json[str(s_year)][str(s_month)]['31']['normal']['successful'],
                self.normal_successful_per_day,
        )
        self.assertEqual(
                response.json[str(s_year + 1)]['1']['1']['rollback']['failed'],
                self.rollback_failed_per_day
        )

    def test_stats_by_date_package_with_unit_day(self):
        """
        Tests /stats/by_date/package
        """
        s_year = self.start_date.year
        s_month = self.start_date.month
        response = self.client.get('/stats/by_date/package?unit=day')

        # Assumes p1 is in every release
        self.assertEqual(
                response.json[str(s_year)][str(s_month)]['31']['p1']['normal']['successful'],
                self.normal_successful_per_day,
        )
        self.assertEqual(
                response.json[str(s_year)][str(s_month)]['31']['p2']['normal']['successful'],
                self.normal_successful_per_day,
        )
        self.assertEqual(
                response.json[str(s_year + 1)]['1']['1']['p1']['rollback']['failed'],
                self.rollback_failed_per_day
        )



from __future__ import print_function
import logging
import sys

__author__ = 'alforbes'


class OrloError(Exception):
    status_code = 500

    def __init__(self, message, status_code=None, payload=None):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload

    def to_dict(self):
        rv = dict(self.payload or ())
        rv['message'] = self.message
        return rv


class InvalidUsage(OrloError):
    status_code = 400


class DatabaseError(OrloError):
    status_code = 500


class OrloWorkflowError(OrloError):
    status_code = 400


class OrloAuthError(OrloError):
    status_code = 401


class OrloStartupError(Exception):
    def __init__(self, message):
        Exception.__init__(self)
        print("Startup Error: " + message)


class OrloConfigError(Exception):
    def __init__(self, message):
        Exception.__init__(self)
        print("Configuration Error: " + message)


class OrloDeployError(OrloError):
    status_code = 500



#!/usr/bin/env python
from __future__ import print_function
import argparse
import fileinput
import json
import logging
from collections import OrderedDict
from logging import error, warn, info
from orloclient import OrloClient, Release, Package
import os

__author__ = 'alforbes'

"""
An example deployer. This is used to run tests against, but could also be used
as a basis for Orlo integration.
"""

logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s')


class DeployerError(Exception):
    def __init__(self, message):
        self.message = message
        error(message)


def get_params():
    parser = argparse.ArgumentParser(description="Orlo Test Deployer")
    parser.add_argument('packages', choices=None, const=None, nargs='*',
                        help="Packages to deploy, in format "
                             "package_name=version, e.g test-package=1.0.0")
    args, unknown = parser.parse_known_args()

    _packages = OrderedDict()

    for pkg in args.packages:
        package, version = pkg.split('=')
        _packages[package] = version

    # Fetch metadata from stdin
    s_metadata = ""
    for line in fileinput.input(unknown):
        s_metadata += line

    try:
        _metadata = json.loads(s_metadata)
    except ValueError:
        raise DeployerError("Could not parse json from stdin")

    return _packages, _metadata


def deploy(package, meta=None):
    """
    Dummy deployment function

    :param Package package: Package to deploy
    :param dict meta: Dictionary of metadata (unused in this dummy function)
    :return:
    """
    info("Package start - {}:{}".format(package.name, package.version))
    orlo_client.package_start(package)

    # Do stuff
    # Determining success status is up to you
    success = True

    info("Package stop - {}:{}".format(package.name, package.version))
    orlo_client.package_stop(package, success=success)

    return success


if __name__ == "__main__":
    if os.getenv('ORLO_URL'):
        orlo_url = os.environ['ORLO_URL']
    else:
        # This deployer is only every supposed to accept releases from Orlo
        # Other deployers could use this to detect whether they are being
        # invoked by Orlo
        raise DeployerError("Could not detect ORLO_URL from environment")

    if os.getenv('ORLO_RELEASE'):
        orlo_release_id = os.environ['ORLO_RELEASE']
    else:
        raise DeployerError("Could not detect ORLO_RELEASE in environment")

    logging.info(
        "Environment: \n" +
        json.dumps(os.environ, indent=2, default=lambda o: o.__dict__))

    # Fetch packages and metadata. Packages is not used, it is just to
    # demonstrate they are passed as arguments
    packages, metadata = get_params()

    logging.info("Stdin: \n" + str(metadata))

    # Create an instance of the Orlo client
    orlo_client = OrloClient(uri=orlo_url)

    # The release is created in Orlo before the deployer is invoked, so fetch
    # it here. If you prefer, you can to do the release creation within your
    # deployer and use Orlo only for receiving data
    release = orlo_client.get_release(orlo_release_id)

    # While we fetch Packages using the Orlo client, they are passed on the
    # CLI as well, which is useful for non-python deployers
    info("Fetching packages from Orlo")
    if not release.packages:
        raise DeployerError("No packages to deploy")

    info("Starting Release")
    for pkg in release.packages:
        info("Deploying {}".format(pkg.name))
        deploy(pkg, meta=metadata)

    info("Finishing Release")
    orlo_client.release_stop(release)

    info("Done.")



from __future__ import print_function, unicode_literals
import arrow
import json
from test_orm import OrloDbTest
import orlo.queries
import orlo.exceptions
import orlo.stats
from time import sleep
import sqlalchemy.orm
import uuid
from orlo.util import is_uuid

__author__ = 'alforbes'


"""
Test the query functions in queries.py
"""


class OrloQueryTest(OrloDbTest):
    pass


class TestList(OrloQueryTest):
    def test_user_list(self):
        """
        Test that user_list returns a list of all users that have released
        """
        self._create_release(user='userOne')
        self._create_release(user='userTwo')

        result = orlo.queries.user_list().all()
        self.assertEqual(len(result), 2)
        users = [r[0] for r in result]
        self.assertIn('userOne', users)
        self.assertIn('userTwo', users)

    def test_user_list_with_platform(self):
        """
        Test user_list with a platform filter
        """
        self._create_release(user='userOne', platforms=['platformOne'])
        self._create_release(user='userTwo', platforms=['platformTwo'])

        result = orlo.queries.user_list(platform='platformOne').all()
        self.assertEqual(len(result), 1)
        users = [r[0] for r in result]
        self.assertIn('userOne', users)
        self.assertNotIn('userTwo', users)

    def test_user_list_with_duplicates(self):
        """
        Test that user_list does returns distinct user names
        """
        self._create_release(user='userOne')
        self._create_release(user='userOne')
        self._create_release(user='userTwo')

        result = orlo.queries.user_list().all()
        self.assertEqual(len(result), 2)
        users = [r[0] for r in result]
        self.assertIn('userOne', users)
        self.assertIn('userTwo', users)

    def test_package_list(self):
        """
        Test that package_list returns a list of all packages released
        """
        for _ in range(0, 3):
            rid = self._create_release()
            self._create_package(rid, name='packageOne')
            self._create_package(rid, name='packageTwo')

        result = orlo.queries.package_list().all()
        self.assertEqual(len(result), 2)
        packages = [r[0] for r in result]
        self.assertIn('packageOne', packages)
        self.assertIn('packageTwo', packages)

    def test_package_list_with_platform(self):
        """
        Test package_list with a platform filter
        """
        rid1 = self._create_release(platforms=['platformOne'])
        self._create_package(rid1, name='packageOne')

        rid2 = self._create_release(platforms=['platformTwo'])
        self._create_package(rid2, name='packageTwo')

        result = orlo.queries.package_list(platform='platformOne').all()
        self.assertEqual(len(result), 1)
        packages = [r[0] for r in result]
        self.assertIn('packageOne', packages)
        self.assertNotIn('packageTwo', packages)

    def test_platform_list(self):
        """
        Test that platform_list returns a list of all platforms that have been released to
        """
        self._create_release(platforms=['platformOne', 'platformTwo'])
        result = orlo.queries.platform_list().all()

        self.assertEqual(len(result), 2)
        platforms = [r[0] for r in result]
        self.assertIn('platformOne', platforms)
        self.assertIn('platformTwo', platforms)


class TestSummary(OrloQueryTest):
    def test_user_summary(self):
        """
        Test that user_summary returns the expected output
        """
        for _ in range(0, 3):
            self._create_release(user='userOne')
        for _ in range(0, 2):
            self._create_release(user='userTwo')

        result = orlo.queries.user_summary().all()
        self.assertEqual(len(result), 2)

        for user, count in result:
            if user == 'userOne':
                self.assertEqual(count, 3)
            elif user == 'userTwo':
                self.assertEqual(count, 2)
            else:
                raise Exception('Unexpected user: {}'.format(str(user)))

    def test_user_summary_with_platform(self):
        """
        Test user_summary with a platform filter
        """
        for _ in range(0, 3):
            self._create_release(user='userOne', platforms=['platformOne'])
        for _ in range(0, 2):
            self._create_release(user='userTwo', platforms=['platformTwo'])

        result = orlo.queries.user_summary(platform='platformOne').all()
        self.assertEqual(len(result), 1)

        for user, count in result:
            if user == 'userOne':
                self.assertEqual(count, 3)
            else:
                raise Exception('Unexpected user: {}'.format(str(user)))

    def test_platform_summary(self):
        """
        Test platform_summary returns a summary
        """

        for _ in range(0, 3):
            self._create_release(platforms=['platformOne', 'platformTwo'])
        for _ in range(0, 2):
            self._create_release(platforms=['platformTwo', 'platformThree'])

        result = orlo.queries.platform_summary().all()
        for platform, count in result:
            if platform == 'platformOne':
                self.assertEqual(count, 3)
            elif platform == 'platformTwo':
                self.assertEqual(count, 5)
            elif platform == 'platformThree':
                self.assertEqual(count, 2)
            else:
                raise Exception('Unexpected platform: {}'.format(str(platform)))

    def test_package_summary(self):
        """
        Test package_summary
        """
        rid1 = self._create_release()
        self._create_package(rid1, name='packageOne')
        self._create_package(rid1, name='packageTwo')

        rid2 = self._create_release()
        self._create_package(rid2, name='packageOne')
        self._create_package(rid2, name='packageTwo')

        result = orlo.queries.package_summary().all()

        self.assertEqual(len(result), 2)
        packages = [r[0] for r in result]
        self.assertIn('packageOne', packages)
        self.assertIn('packageTwo', packages)

    def test_package_summary_returns_query(self):
        rid1 = self._create_release()
        self._create_package(rid1, name='packageOne')

        result = orlo.queries.package_summary()
        self.assertIsInstance(result, sqlalchemy.orm.query.Query)

    def test_package_summary_with_platform(self):
        """
        Test package_summary
        """
        rid1 = self._create_release(platforms=['platformOne'])
        self._create_package(rid1, name='packageOne')

        rid2 = self._create_release(platforms=['platformTwo'])
        self._create_package(rid2, name='packageTwo')

        result = orlo.queries.package_summary(platform='platformOne').all()

        self.assertEqual(len(result), 1)
        packages = [r[0] for r in result]
        self.assertIn('packageOne', packages)

    def test_package_summary_with_stime(self):
        """
        Test package_summary with stime filter inclusive (hour in past)
        """
        rid1 = self._create_release()
        self._create_package(rid1, name='packageOne')

        hour_ago = arrow.utcnow().replace(hours=-1)
        result = orlo.queries.package_summary(stime=hour_ago).all()

        self.assertEqual(len(result), 1)
        packages = [r[0] for r in result]
        self.assertIn('packageOne', packages)

    def test_package_summary_with_stime_negative(self):
        """
        Test package_summary with stime filter exclusive (hour in future)
        """
        rid1 = self._create_release()
        self._create_package(rid1, name='packageOne')

        hour_future = arrow.utcnow().replace(hours=+1)
        result = orlo.queries.package_summary(stime=hour_future).all()

        self.assertEqual(len(result), 0)

    def test_package_summary_with_ftime(self):
        """
        Test package_summary with ftime filter inclusive (hour in future)
        """
        rid1 = self._create_release()
        self._create_package(rid1, name='packageOne')

        hour_future = arrow.utcnow().replace(hours=+1)
        result = orlo.queries.package_summary(ftime=hour_future).all()

        self.assertEqual(len(result), 1)
        packages = [r[0] for r in result]
        self.assertIn('packageOne', packages)

    def test_package_summary_with_ftime_negative(self):
        """
        Test package_summary with ftime filter exclusive (hour in past)
        """
        rid1 = self._create_release()
        self._create_package(rid1, name='packageOne')

        hour_past = arrow.utcnow().replace(hours=-1)
        result = orlo.queries.package_summary(ftime=hour_past).all()

        self.assertEqual(len(result), 0)

    def test_package_versions(self):
        """
        Test package_versions

        In this test, we create two releases. packageOne succeeds in both but packageTwo fails
        in the second, therefore the current version for packageOne should be the second release,
        but packageTwo should remain with the first version
        """
        rid1 = self._create_release(platforms=['platformOne'])
        pid1 = self._create_package(rid1, name='packageOne', version='1.0.1')
        pid2 = self._create_package(rid1, name='packageTwo', version='2.0.1')
        self._start_package(pid1)
        self._stop_package(pid1)
        self._start_package(pid2)
        self._stop_package(pid2)
        sleep(0.1)  # To ensure some time separation
        rid2 = self._create_release(platforms=['platformOne'])
        pid1 = self._create_package(rid2, name='packageOne', version='1.0.2')
        pid2 = self._create_package(rid2, name='packageTwo', version='2.0.2')
        self._start_package(pid1)
        self._stop_package(pid1)
        self._start_package(pid2)
        self._stop_package(pid2, success=False)

        result = orlo.queries.package_versions().all()
        self.assertEqual(len(result), 2)  # Two entries, packageOne/Two
        versions = [(p, v) for p, v in result]  # strip out the time
        # Correct versions:
        self.assertIn(('packageOne', '1.0.2'), versions)
        self.assertIn(('packageTwo', '2.0.1'), versions)

    def test_package_versions_with_platform(self):
        """
        Test package_versions excludes non-specified platforms
        """
        self._create_finished_release()  # this release should not appear in result
        rid1 = self._create_release(platforms=['specific_platform'])
        pid1 = self._create_package(rid1, name='packageOne', version='1.0.1')
        self._start_package(pid1)
        self._stop_package(pid1)

        result = orlo.queries.package_versions(platform='specific_platform').all()

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0][0], 'packageOne')

    def test_package_versions_with_rollback(self):
        """
        Test package_versions with a rollback

        We should return the versions that were rolled back to and not exclude them
        """
        rid1 = self._create_release(platforms=['platformOne'])
        pid1 = self._create_package(rid1, name='packageOne', version='1.0.2')
        self._start_package(pid1)
        self._stop_package(pid1)
        sleep(0.1)  # To ensure some time separation
        rid2 = self._create_release(platforms=['platformOne'])
        pid1 = self._create_package(rid2, name='packageOne', version='1.0.1', rollback=True)
        self._start_package(pid1)
        self._stop_package(pid1)

        result = orlo.queries.package_versions().all()
        self.assertEqual(len(result), 1)  # One entry, packageOne
        versions = [(p, v) for p, v in result]  # strip out the time
        # Correct versions:
        self.assertIn(('packageOne', '1.0.1'), versions)


class TestInfo(OrloQueryTest):
    """
    Test the _info functions
    """

    def _create_test_package(self):
        rid = self._create_release(platforms=['platformOne'])
        pid = self._create_package(rid, name='packageOne', version='1.0.1')
        return pid

    def test_returns_query(self):
        """
        Assert that package_info should return a query
        """
        self._create_test_package()
        result = orlo.queries.package_info('packageOne')
        self.assertIsInstance(result, sqlalchemy.orm.query.Query)


class TestCountReleases(OrloQueryTest):
    """
    Parent class for testing the CountReleases function

    By subclassing it and overriding ARGS, we can test different combinations of
    arguments with the same test code.

    INCLUSIVE_ARGS represents a set of arguments that will match the releases
    created (see the functions in OrloQueryTest for what those are)
    EXCLUSIVE_ARGS represents a set of arguments that will not match any
    releases created, i.e. should return a count of zero

    This parent class has tests that should be the same result no matter what
    the arguments (except the exclusive case which must always a count of zero
    so we define it here)
    """

    INCLUSIVE_ARGS = {}  # Args we are testing, result should include these
    EXCLUSIVE_ARGS = {'user': 'non-existent'}  # Args which result in zero

    def test_count_releases(self):
        """
        Test count_releases in the simplest case
        """
        self._create_finished_release()
        result = orlo.queries.count_releases(**self.INCLUSIVE_ARGS).all()
        self.assertEqual(1, result[0][0])

    def test_count_releases_length(self):
        """
        Test that the function returns the correct number of items
        """
        for _ in range(0, 3):
            self._create_finished_release()
        # Should still only return one result (a count)
        result = orlo.queries.count_releases(**self.INCLUSIVE_ARGS).all()
        self.assertEqual(1, len(result))

    def test_count_releases_inclusive(self):
        """
        Test count_releases returns a count of one when filtering on INCLUSIVE_ARGS
        """
        self._create_finished_release()
        result = orlo.queries.count_releases(**self.INCLUSIVE_ARGS).all()
        self.assertEqual(1, result[0][0])

    def test_count_releases_exclusive(self):
        """
        Test count_releases returns a count of zero when filtering on EXCLUSIVE_ARGS
        """
        self._create_finished_release()
        result = orlo.queries.count_releases(**self.EXCLUSIVE_ARGS).all()
        self.assertEqual(0, result[0][0])


class CountReleasesStatus(TestCountReleases):
    """
    Test count_releases when filtering on status

    Note that this is a special case, as whether we consider a release
    successful or failed depends on the _combination_ of the package statuses.

    Will also run parent class tests with arguments below
    """
    INCLUSIVE_ARGS = {'status': 'SUCCESSFUL'}
    EXCLUSIVE_ARGS = {'status': 'FAILED'}

    def test_count_releases_excludes_failed(self):
        """
        Test that count_releases only returns successful releases

        By adding a failed one
        """

        for _ in range(0, 3):
            # 3 complete, successful releases
            self._create_finished_release()

        # And a fourth with one failed package (should not be present)
        rid = self._create_release()
        pid1 = self._create_package(rid)
        pid2 = self._create_package(rid)
        self._start_package(pid1)
        self._start_package(pid2)
        self._stop_package(pid1, success=True)
        self._stop_package(pid2, success=False)
        self._stop_release(rid)

        result = orlo.queries.count_releases(**self.INCLUSIVE_ARGS).all()
        self.assertEqual(3, result[0][0])

    def test_count_releases_invalid_status(self):
        """
        Test that we raise OrloError when proving an invalid status

        As this is an enum it's more useful to raise an error than simply
        return a zero count
        """
        with self.assertRaises(orlo.exceptions.OrloError):
            query = orlo.queries.count_releases(status='BAD_STATUS_FOOBAR')
            query.all()

    def test_count_releases_in_progress(self):
        """
        Test that we correctly return the number of releases in progress
        """
        for _ in range(0, 3):
            # 3 complete, successful releases
            self._create_finished_release()

        # And a fourth with the packages still in progress
        rid = self._create_release()
        pid1 = self._create_package(rid)
        pid2 = self._create_package(rid)
        self._start_package(pid1)
        self._start_package(pid2)

        result = orlo.queries.count_releases(status='IN_PROGRESS').all()
        self.assertEqual(1, result[0][0])


class CountReleasesUser(TestCountReleases):
    """
    Test count_releases when filtering on user
    """
    INCLUSIVE_ARGS = {'user': 'testuser'}
    EXCLUSIVE_ARGS = {'user': 'bad_foo_user'}


class CountReleasesTeam(TestCountReleases):
    """
    Test count_releases when filtering on team
    """
    INCLUSIVE_ARGS = {'team': 'test team'}
    EXCLUSIVE_ARGS = {'team': 'bad team does not exist'}


class CountReleasesPackage(TestCountReleases):
    """
    Test count_releases when filtering on package
    """
    INCLUSIVE_ARGS = {'package': 'test-package'}
    EXCLUSIVE_ARGS = {'package': 'non-existent-package'}


class CountReleasesPlatform(TestCountReleases):
    """
    Test count_releases when filtering on platform
    """
    INCLUSIVE_ARGS = {'platform': 'test_platform'}
    EXCLUSIVE_ARGS = {'platform': 'non_existent_platform'}


class CountReleasesRollback(TestCountReleases):
    """
    Test count_releases when filtering on rollback

    The args are a little counterintuitive here, but True is the exclusive case
    because the parent class does not create releases with rollback set to True.
    """
    INCLUSIVE_ARGS = {'rollback': False}
    EXCLUSIVE_ARGS = {'rollback': True}

    def test_bad_rollback_param_raises_type_error(self):
        """
        Test that we get a TypeError when passing garbage to the rollback param
        """
        with self.assertRaises(TypeError):
            orlo.queries.count_releases(rollback='foo')

    def test_rollback_true(self):
        """
        Test that we correctly count rollback releases

        The parent class creates only non-rollback releases
        """
        for _ in range(0, 3):
            # 3 complete, non-rollback releases
            self._create_finished_release()

        # And a fourth with all packages rollbacks
        rid = self._create_release()
        self._create_package(rid, rollback=True)
        self._create_package(rid, rollback=True)

        result = orlo.queries.count_releases(rollback=True).all()
        self.assertEqual(1, result[0][0])

    def test_rollback_false(self):
        """
        Test that we exclude rollback releases when setting rollback = False
        """
        rid = self._create_release()
        self._create_package(rid, rollback=True)
        self._create_package(rid, rollback=False)

        result = orlo.queries.count_releases(rollback=False).all()
        self.assertEqual(0, result[0][0])

    def test_partial_rollback(self):
        """
        Test a release is still counted when some packages are not rollbacks
        """
        for _ in range(0, 3):
            # 3 complete, non-rollback releases
            self._create_finished_release()

        # And a fourth with one package rollback, one not
        rid = self._create_release()
        self._create_package(rid, rollback=True)
        self._create_package(rid, rollback=False)

        result = orlo.queries.count_releases(rollback=True).all()
        self.assertEqual(1, result[0][0])


class CountPackagesTest(OrloQueryTest):
    """
    Parent class for testing the CountPackages function

    By subclassing it and overriding ARGS, we can test different combinations
    of arguments with the same test code.

    INCLUSIVE_ARGS represents a set of arguments that will match the packages
    created (see the functions in OrloQueryTest for what those are)
    EXCLUSIVE_ARGS represents a set of arguments that will not match any
    packages created, i.e. should return a count of zero

    This parent class has tests that should be the same result no matter what
    the arguments (except the exclusive case which must always a count of zero
    so we define it here)
    """

    INCLUSIVE_ARGS = {}  # Args we are testing, result should include these
    EXCLUSIVE_ARGS = {'user': 'non-existent'}  # Args which result in zero

    def test_count_packages(self):
        """
        Test count_packages in the simplest case
        """
        self._create_finished_release()
        result = orlo.queries.count_packages(**self.INCLUSIVE_ARGS).all()
        self.assertEqual(1, result[0][0])

    def test_count_packages_length(self):
        """
        Test that the function returns the correct number of items
        """
        for _ in range(0, 3):
            self._create_finished_release()
        # Should still only return one result (a count)
        result = orlo.queries.count_packages(**self.INCLUSIVE_ARGS).all()
        self.assertEqual(1, len(result))

    def test_count_packages_inclusive(self):
        """
        Test count_packages returns count of one filtering on INCLUSIVE_ARGS
        """
        self._create_finished_release()
        result = orlo.queries.count_packages(**self.INCLUSIVE_ARGS).all()
        self.assertEqual(1, result[0][0])

    def test_count_packages_exclusive(self):
        """
        Test count_packages returns a count of zero filtering on EXCLUSIVE_ARGS
        """
        self._create_finished_release()
        result = orlo.queries.count_packages(**self.EXCLUSIVE_ARGS).all()
        self.assertEqual(0, result[0][0])


class CountPackagesStatus(CountPackagesTest):
    """
    Test count_packages when filtering on status

    Note that this is a special case, as whether we consider a release
    successful or failed depends on the _combination_ of the package statuses.

    Will also run parent class tests with arguments below
    """
    INCLUSIVE_ARGS = {'status': 'SUCCESSFUL'}
    EXCLUSIVE_ARGS = {'status': 'FAILED'}


class CountPackagesUser(CountPackagesTest):
    """
    Test count_packages when filtering on user
    """
    INCLUSIVE_ARGS = {'user': 'testuser'}
    EXCLUSIVE_ARGS = {'user': 'bad_foo_user'}


class CountPackagesTeam(CountPackagesTest):
    """
    Test count_packages when filtering on team
    """
    INCLUSIVE_ARGS = {'team': 'test team'}
    EXCLUSIVE_ARGS = {'team': 'bad team does not exist'}


class CountPackagesPlatform(CountPackagesTest):
    """
    Test count_packages when filtering on platform
    """
    INCLUSIVE_ARGS = {'platform': 'test_platform'}
    EXCLUSIVE_ARGS = {'platform': 'non_existent_platform'}


class CountPackagesRollback(CountPackagesTest):
    """
    Test count_packages when filtering on rollback
    """
    INCLUSIVE_ARGS = {'rollback': False}
    EXCLUSIVE_ARGS = {'rollback': True}

    def test_rollback_true(self):
        """
        Test count_packages returns a count of one filtering on INCLUSIVE_ARGS
        """
        rid = self._create_release()
        self._create_package(rid, rollback=True)
        self._create_package(rid, rollback=True)

        result = orlo.queries.count_packages(rollback=True).all()
        self.assertEqual(2, result[0][0])

    def test_rollback_false(self):
        """
        Test count_packages returns a count of one filtering on INCLUSIVE_ARGS
        """
        rid = self._create_release()
        self._create_package(rid, rollback=True)
        self._create_package(rid, rollback=False)

        result = orlo.queries.count_packages(rollback=False).all()
        self.assertEqual(1, result[0][0])


class ReleasesTest(OrloQueryTest):
    """
    Test the releases method
    """

    def test_releases_with_bad_limit(self):
        """
        Test releases raises InvalidUsage when limit is not an int
        """
        args = {
            'limit': 'bad_limit',
        }
        with self.assertRaises(orlo.exceptions.InvalidUsage):
            orlo.queries.releases(**args)

    def test_releases_with_bad_offset(self):
        """
        Test releases raises InvalidUsage when offset is not an int
        """
        args = {
            'offset': 'bad_offset',
        }
        with self.assertRaises(orlo.exceptions.InvalidUsage):
            orlo.queries.releases(**args)


class GetPackageTest(OrloQueryTest):
    """
    Test the packages method
    """

    def test_get_package_returns_query(self):
        """
        Test that we return a Query object
        """
        result = orlo.queries.get_package('foo')
        self.assertIsInstance(result, sqlalchemy.orm.query.Query)

    def test_get_package_works(self):
        """
        Test that the query will give us the right package
        """
        rid = self._create_release()
        pid = self._create_package(rid, rollback=False)
        result = orlo.queries.get_package(pid).all()

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].id, pid)


class PackagesTests(OrloQueryTest):
    """
    Test the queries.packages method
    """
    def test_packages_returns_query(self):
        """
        Test that
        :return:
        """
        result = orlo.queries.packages()
        self.assertIsInstance(result, sqlalchemy.orm.query.Query)

    def test_packages_works(self):
        """
        Test that query returned by get_package works
        """
        rid = self._create_release()
        pid = self._create_package(rid, rollback=False)
        result = orlo.queries.packages().all()
        self.assertEqual(result[0].id, pid)



from flask import jsonify, request, Response, json, g
from orlo import app, queries, config
from orlo.exceptions import InvalidUsage
from orlo.user_auth import token_auth
from orlo.orm import db, Release, Package, PackageResult, ReleaseNote, \
    ReleaseMetadata, Platform
from orlo.util import validate_request_json, create_release, \
    validate_release_input, validate_package_input, fetch_release, \
    create_package, fetch_package, stream_json_list, str_to_bool, is_uuid
from orlo.deploy import ShellDeploy
from orlo.user_auth import conditional_auth

security_enabled = config.getboolean('security', 'enabled')


@app.route('/releases', methods=['POST'])
@conditional_auth(conditional_auth(token_auth.token_required))
def post_releases():
    """
    Create a release - the first step in a deployment

    :<json string user: User that is performing the release
    :<json string team: The development team responsible for this release
    :<json array platforms: List of platforms receiving the release
    :<json array references: List of external references, e.g. Jira ticket
    :>json string id: UUID reference to the created release
    :reqheader Content-Type: Must be application/json
    :status 200: Release was created successfully
    :status 400: Invalid request

    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST \\
        http://127.0.0.1/releases \\
        -d '{"note": "blah", "platforms": ["site1"], "references": ["ticket"],
        "team": "A-Team", "user": "aforbes"}'
    """
    validate_release_input(request)
    release = create_release(request)

    if request.json.get('note'):
        release_note = ReleaseNote(release.id, request.json.get('note'))
        db.session.add(release_note)

    if request.json.get('metadata'):
        for key, value in request.json.get('metadata').items():
            metadata = ReleaseMetadata(release.id, key, value)
            db.session.add(metadata)

    app.logger.info(
        'Create release {}, references: {}, platforms: {}'.format(
            release.id, release.notes, release.references, release.platforms,
            release.metadata)
    )

    release.start()

    db.session.add(release)
    db.session.commit()

    return jsonify(id=release.id)


@app.route('/releases/<release_id>/packages', methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_packages(release_id):
    """
    Add a package to a release

    :param string release_id: UUID of the release to add the package to
    :<json string name: Name of the package
    :<json string version: Version of the package
    :<json boolean rollback: Whether this package deploy is a rollback
    :>json string id: UUID reference to the created package
    :reqheader Content-Type: Must be application/json
    :status 200: Package was added to the release successfully
    :status 400: Invalid request
    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST http://127.0.0.1/releases/${RELEASE_ID}/packages \\
        -d '{"name": "test-package", "version": "1.0.1"}'
    """
    validate_package_input(request, release_id)

    release = fetch_release(release_id)
    package = create_package(release.id, request)

    app.logger.info(
        'Create package {}, release {}, name {}, version {}'.format(
            package.id, release.id, request.json['name'],
            request.json['version']))

    db.session.add(package)
    db.session.commit()

    return jsonify(id=package.id)


@app.route('/releases/<release_id>/packages/<package_id>/results',
           methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_results(release_id, package_id):
    """
    Post the results of a package release

    :param string release_id: Release UUID
    :param string package_id: Package UUID
    :<json string content: Free text field to store what you wish
    :status 204: Package results added successfully
    """
    results = PackageResult(package_id, str(request.json))
    app.logger.info("Post results, release {}, package {}".format(
        release_id, package_id))
    db.session.add(results)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/deploy', methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_releases_deploy(release_id):
    """
    Deploy a Release

    :param string release_id: Release UUID

    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST http://127.0.0.1/releases/${RELEASE_ID}/deploy
    """
    release = fetch_release(release_id)
    app.logger.info("Release start, release {}".format(release_id))

    release.start()
    db.session.add(release)
    db.session.commit()

    # TODO call deploy Class start Method, i.e. pure python rather than shell
    deploy = ShellDeploy(release)
    output = deploy.start()
    return jsonify(output), 200


@app.route('/releases/<release_id>/start', methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_releases_start(release_id):
    """
    Indicate that a release is starting

    :param release_id:
    :return:
    """
    release = fetch_release(release_id)
    app.logger.info("Release start, release {}".format(release_id))
    release.start()

    db.session.add(release)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/stop', methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_releases_stop(release_id):
    """
    Indicate that a release has finished

    This should be called after all packages have also been "stopped".
    In future it may stop any un-stopped packages.

    :param string release_id: Release UUID

    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST http://127.0.0.1/releases/${RELEASE_ID}/stop
    """
    release = fetch_release(release_id)
    # TODO check that all packages have been finished
    app.logger.info("Release stop, release {}".format(release_id))
    release.stop()

    db.session.add(release)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/packages/<package_id>/start',
           methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_packages_start(release_id, package_id):
    """
    Indicate that a package has started deploying

    :param string release_id: Release UUID
    :param string package_id: Package UUID
    :status 204:

    **Example curl**:

    .. sourcecode:: shell

        curl -X POST http://127.0.0.1/releases/${RELEASE_ID}/packages/${
        PACKAGE_ID}/start
    """
    package = fetch_package(release_id, package_id)
    app.logger.info("Package start, release {}, package {}".format(
        release_id, package_id))
    package.start()

    db.session.add(package)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/packages/<package_id>/stop',
           methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_packages_stop(release_id, package_id):
    """
    Indicate that a package has finished deploying

    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST \\
        http://127.0.0.1/releases/${RELEASE_ID}/packages/${PACKAGE_ID}/stop \\
        -d '{"success": "true"}'

    :param string package_id: Package UUID
    :param string release_id: Release UUID
    """
    validate_request_json(request)
    success = request.json.get('success') in [True, 'True', 'true', '1']

    package = fetch_package(release_id, package_id)
    app.logger.info("Package stop, release {}, package {}, success {}".format(
        release_id, package_id, success))
    package.stop(success=success)

    db.session.add(package)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/notes', methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_releases_notes(release_id):
    """
    Add a note to a release

    :param string release_id: Release UUID
    :query string text: Text
    :return:
    """
    validate_request_json(request)
    text = request.json.get('text')
    if not text:
        raise InvalidUsage("Must include text in posted document")

    note = ReleaseNote(release_id, text)
    app.logger.info("Adding note to release {}".format(release_id))
    db.session.add(note)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/metadata', methods=['POST'])
@conditional_auth(token_auth.token_required)
def post_releases_metadata(release_id):
    """
    Add metadata to a release

    :param string release_id: Release UUID
    :query string text: Text
    :return:
    """
    validate_request_json(request)
    meta = request
    if not meta:
        raise InvalidUsage(
            "Must include metadata in posted document: es {\"key\" : "
            "\"value\"}")

    for key, value in request.json.items():
        app.logger.info("Adding Metadata to release {}".format(release_id))
        metadata = ReleaseMetadata(release_id, key, value)
        db.session.add(metadata)

    db.session.commit()
    return '', 204


@app.route('/releases', methods=['GET'])
@app.route('/releases/<release_id>', methods=['GET'])
def get_releases(release_id=None):
    """
    Return a list of releases to the client, filters optional

    :param string release_id: Optionally specify a single release UUID to
        fetch. This does not disable filters.
    :query bool desc: Normally results are returned ordered by stime
        ascending, setting desc to true will reverse this and sort by stime
        descending
    :query int limit: Limit the results by int
    :query int offset: Offset the results by int
    :query string package_name: Filter releases by package name
    :query string user: Filter releases by user the that performed the release
    :query string platform: Filter releases by platform
    :query string stime_before: Only include releases that started before \
        timestamp given
    :query string stime_after: Only include releases that started after \
        timestamp given
    :query string ftime_before: Only include releases that finished before \
        timestamp given
    :query string ftime_after: Only include releases that finished after \
        timestamp given
    :query string team: Filter releases by team
    :query string status: Filter by release status. This field is calculated \
        from the package status, see special note below.
    :query int duration_lt: Only include releases that took less than (int) \
        seconds
    :query int duration_gt: Only include releases that took more than (int) \
        seconds
    :query boolean package_rollback: Filter on whether or not the releases \
        contain a rollback
    :query string package_name: Filter by package name
    :query string package_version: Filter by package version
    :query int package_duration_gt: Filter by packages of duration greater than
    :query int package_duration_lt: Filter by packages of duration less than
    :query string package_status: Filter by package status. Valid statuses are:\
         "NOT_STARTED", "IN_PROGRESS", "SUCCESSFUL", "FAILED"

    **Note for time arguments**:
        The timestamp format you must use is specified in /etc/orlo/orlo.ini.
        All times are UTC.

    **Note on status**:
        The release status is calculated from the packages it contains. The
        possible values are the same as a package. For a release to be
        considered "SUCCESSFUL" or "NOT_STARTED", all packages must have this
        value. If any one package has the value "IN_PROGRESS" or "FAILED",
        that status applies to the whole release, with "FAILED" overriding
        "IN_PROGRESS".

    """

    booleans = ('rollback', 'package_rollback',)

    if release_id:  # Simple, just fetch one release
        if not is_uuid(release_id):
            raise InvalidUsage("Release ID given is not a valid UUID")
        query = queries.get_release(release_id)
    elif len([x for x in request.args.keys()]) == 0:
        raise InvalidUsage("Please specify a filter. See "
                           "http://orlo.readthedocs.org/en/latest/rest.html"
                           "#get--releases for more info")
    else:  # Bit more complex
        # Flatten args, as the ImmutableDict puts some values in a list when
        # expanded
        args = {}
        for k in request.args.keys():
            if k in booleans:
                args[k] = str_to_bool(request.args.get(k))
            else:
                args[k] = request.args.get(k)
        query = queries.releases(**args)

    # Execute eagerly to avoid confusing stack traces within the Response on
    # error
    db.session.execute(query)

    return Response(stream_json_list('releases', query),
                    content_type='application/json')



from orlo import app
from orlo.config import config
from orlo.exceptions import OrloAuthError
from flask_httpauth import HTTPBasicAuth
from flask_tokenauth import TokenAuth, TokenManager
from flask import request, jsonify, g, make_response
from werkzeug.security import generate_password_hash, check_password_hash
from itsdangerous import (TimedJSONWebSignatureSerializer
                          as Serializer, BadSignature, SignatureExpired)
from functools import update_wrapper, wraps
import ldap


# initialization
user_auth = HTTPBasicAuth()
token_auth = TokenAuth(config.get('security', 'secret_key'))
token_manager = TokenManager(secret_key=config.get('security', 'secret_key'))


class conditional_auth(object):
    """
    Decorator which wraps a decorator, to only apply it if auth is enabled
    """
    def __init__(self, decorator):
        self.decorator = decorator
        update_wrapper(self, decorator)

    def __call__(self, func):
        """
        Call method
        """
        @wraps(func)
        def wrapped(*args, **kwargs):
            """
            Wrapped method
            """
            if config.getboolean('security', 'enabled'):
                app.logger.debug("Security enabled")
                return self.decorator(func)(*args, **kwargs)
            else:
                app.logger.debug("Security disabled")
                return func(*args, **kwargs)
        return wrapped


class User(object):
    def __init__(self, username, password):
        self.username = username
        self.password_hash = self.hash_password(password)
        self.confirmed = False

    def hash_password(self, password):
        self.password_hash = generate_password_hash(password)

    def verify_password(self, password):
        rc = check_password_hash(self.password_hash, password)
        return rc

    def generate_auth_token(self, expiration=3600):
        secret_key = config.get('security', 'secret_key')
        s = Serializer(secret_key, expires_in=expiration)
        return s.dumps({'id': self.username})

    @staticmethod
    def verify_auth_token(token):
        app.logger.debug("Verify auth token called")
        s = Serializer(config.get('security', 'secret_key'))
        try:
            data = s.loads(token)
        except SignatureExpired:
            return None  # valid token, but expired
        except BadSignature:
            return None  # invalid token
        user = data['id']
        return user


@user_auth.verify_password
def verify_password(username_or_token=None, password=None):
    # first try to authenticate by token
    user = User.verify_auth_token(username_or_token)
    if user:
        set_current_user_as(user)
        return True
    elif not user:
        password_file_user = verify_password_file(username_or_token, password)
        if password_file_user:
            set_current_user_as(username_or_token)
            return True
        else:
            ldap_user = verify_ldap_access(username_or_token, password)
            if ldap_user:
                set_current_user_as(username_or_token)
                return True
            else:
                return False


@token_auth.verify_token
def verify_token(token=None):
    app.logger.debug("Verify token called")
    if not token:
        return False
    token_user = token_manager.verify(token)
    if token_user:
        set_current_user_as(token_user)
        return True


def set_current_user_as(user):
    if not g.get('current_user'):
        app.logger.debug('Setting current user to: {}'.format(user))
        g.current_user = user


def verify_ldap_access(username, password):
    app.logger.debug("Verify ldap called")
    try:
        ldap_server = config.get('security', 'ldap_server')
        ldap_port = config.get('security', 'ldap_port')
        user_base_dn = config.get('security', 'user_base_dn')
        l = ldap.initialize('ldap://{}:{}'.format(ldap_server, ldap_port))
        ldap_user = "uid=%s,%s" % (username, user_base_dn)
        l.protocol_version = ldap.VERSION3
        l.simple_bind_s(ldap_user, password)
        return True
    except ldap.LDAPError:
        return False


def verify_password_file(username=None, password=None):
    app.logger.debug("Verify password file called")
    password_file = config.get('security', 'passwd_file')
    with open(password_file) as f:
        for line in f:
            line = line.strip()
            user = line.split(':')[0]
            if not user == username:
                continue
            # found user return password
            elif user == username:
                app.logger.debug("Found user {} in file".format(username))
                pw = ':'.join(line.split(':')[1:])
                checked_password = check_password_hash(pw, password)
                if checked_password:
                    return True
                else:
                    return False


@user_auth.error_handler
@token_auth.error_handler
def auth_error():
    """
    Authentication error
    """
    response = jsonify({'error': 'not authorized'})
    response.status_code = 401
    return response


@app.route('/token')
@conditional_auth(user_auth.login_required)
def get_token():
    """
    Get a token
    """
    ttl = config.getint('security', 'token_ttl')
    token = token_manager.generate(g.current_user, ttl)
    return jsonify({
        'token': token.decode('ascii'),
        'duration': config.get('security', 'token_ttl')
    })



from __future__ import print_function, unicode_literals
import arrow
import orlo.queries
import orlo.exceptions
import orlo.stats
from test_orm import OrloDbTest

__author__ = 'alforbes'


class OrloStatsTest(OrloDbTest):
    """
    Parent class for the stats tests
    """
    ARGS = {
        'stime_gt': arrow.utcnow().replace(hours=-1),
        'stime_lt': arrow.utcnow().replace(hours=+1)
    }

    def setUp(self):
        super(OrloDbTest, self).setUp()

        for r in range(0, 7):
            self._create_finished_release()


class TestGeneral(OrloStatsTest):
    """
    Testing the shared stats functions
    """

    def test_append_tree_recursive(self):
        """
        Test that append_tree_recursive returns a properly structured dictionary
        """
        tree = {}
        nodes = ['parent', 'child']
        orlo.stats.append_tree_recursive(tree, nodes[0], nodes)
        self.assertEqual(tree, {'parent': {'child': 1}})

    def test_append_tree_recursive_adds(self):
        """
        Test that append_tree_recursive correctly adds one when called on the same path
        """
        tree = {}
        nodes = ['parent', 'child']
        orlo.stats.append_tree_recursive(tree, nodes[0], nodes)
        orlo.stats.append_tree_recursive(tree, nodes[0], nodes)
        self.assertEqual(tree, {'parent': {'child': 2}})


class TestReleaseTime(OrloStatsTest):
    """
    Test stats.releases_by_time
    """

    def test_release_time_month(self):
        """
        Test stats.add_objects_by_time_to_dict by month
        """
        result = orlo.stats.releases_by_time('month', **self.ARGS)
        year = str(arrow.utcnow().year)
        month = str(arrow.utcnow().month)
        self.assertEqual(7, result[year][month]['normal']['successful'])

    def test_release_time_week(self):
        """
        Test stats.add_objects_by_time_to_dict by week
        """
        result = orlo.stats.releases_by_time('week', **self.ARGS)
        year, week, day = arrow.utcnow().isocalendar()
        self.assertEqual(7, result[str(year)][str(week)]['normal']['successful'])

    def test_release_time_year(self):
        """
        Test stats.add_objects_by_time_to_dict by year
        """
        result = orlo.stats.releases_by_time('year', **self.ARGS)
        year = str(arrow.utcnow().year)
        self.assertEqual(7, result[str(year)]['normal']['successful'])

    def test_release_time_day(self):
        """
        Test stats.add_objects_by_time_to_dict by day
        """
        result = orlo.stats.releases_by_time('day', **self.ARGS)
        year = str(arrow.utcnow().year)
        month = str(arrow.utcnow().month)
        day = str(arrow.utcnow().day)
        self.assertEqual(
                7,
                result[year][month][day]['normal']['successful'],
        )

    def test_release_time_hour(self):
        """
        Test stats.add_objects_by_time_to_dict by hour
        """
        result = orlo.stats.releases_by_time('hour', **self.ARGS)
        year = str(arrow.utcnow().year)
        month = str(arrow.utcnow().month)
        day = str(arrow.utcnow().day)
        hour = str(arrow.utcnow().hour)
        self.assertEqual(
                7,
                result[year][month][day][hour]['normal']['successful'],
        )

    def test_release_time_with_only_this_unit(self):
        """
        Test stats.add_objects_by_time_to_dict with only_this_unit

        Should break down by only the unit given
        """
        result = orlo.stats.releases_by_time('hour', summarize_by_unit=True, **self.ARGS)
        hour = str(arrow.utcnow().hour)
        self.assertEqual(
                7,
                result[hour]['normal']['successful'],
        )

    def test_release_time_with_unit_day(self):
        pass


class TestPackageTime(OrloStatsTest):
    """
    Test stats.packages_by_time
    """
    def test_package_time_month(self):
        """
        Test stats.add_objects_by_time_to_dict by month
        """
        result = orlo.stats.packages_by_time('month', **self.ARGS)
        year = str(arrow.utcnow().year)
        month = str(arrow.utcnow().month)
        self.assertEqual(7, result[year][month]['test-package']['normal']['successful'])

    def test_package_time_week(self):
        """
        Test stats.add_objects_by_time_to_dict by week
        """
        result = orlo.stats.packages_by_time('week', **self.ARGS)
        year, week, day = arrow.utcnow().isocalendar()
        self.assertEqual(7, result[str(year)][str(week)]['test-package']['normal']['successful'])

    def test_package_time_year(self):
        """
        Test stats.add_objects_by_time_to_dict by year
        """
        result = orlo.stats.packages_by_time('year', **self.ARGS)
        year = str(arrow.utcnow().year)
        self.assertEqual(7, result[str(year)]['test-package']['normal']['successful'])

    def test_package_time_day(self):
        """
        Test stats.add_objects_by_time_to_dict by day
        """
        result = orlo.stats.packages_by_time('day', **self.ARGS)
        year = str(arrow.utcnow().year)
        month = str(arrow.utcnow().month)
        day = str(arrow.utcnow().day)
        self.assertEqual(
                7,
                result[year][month][day]['test-package']['normal']['successful'],
        )

    def test_package_time_hour(self):
        """
        Test stats.add_objects_by_time_to_dict by hour
        """
        result = orlo.stats.packages_by_time('hour', **self.ARGS)
        year = str(arrow.utcnow().year)
        month = str(arrow.utcnow().month)
        day = str(arrow.utcnow().day)
        hour = str(arrow.utcnow().hour)
        self.assertEqual(
                7,
                result[year][month][day][hour]['test-package']['normal']['successful'],
        )

    def test_package_time_with_only_this_unit(self):
        """
        Test stats.add_objects_by_time_to_dict with only_this_unit

        Should break down by only the unit given
        """
        result = orlo.stats.packages_by_time('hour', summarize_by_unit=True, **self.ARGS)
        hour = str(arrow.utcnow().hour)
        self.assertEqual(
                7,
                result[hour]['test-package']['normal']['successful'],
        )

    def test_package_time_with_unit_day(self):
        pass



from __future__ import print_function
from test_route_base import OrloTest
from random import randrange
from orlo.orm import db
from orlo.orm import Release, Package, PackageResult, Platform
from orlo import app
from sqlalchemy.orm import exc
import arrow
import datetime
import uuid
from six import string_types
from test_base import ReleaseDbUtil

__author__ = 'alforbes'

'''
Tests for the database / orm layer
'''


class OrloDbTest(OrloTest, ReleaseDbUtil):
    """
    Base test class for tests that need to manipulate data without using the http API
    """


class TestFields(OrloDbTest):
    def setUp(self):
        super(OrloDbTest, self).setUp()

        for r in range(0, 3):
            self._create_finished_release()

    def test_release_types(self):
        """
        Test the types returned by Release objects are OK
        """
        r = db.session.query(Release).first()
        self.assertIsInstance(r.id, uuid.UUID)
        self.assertIs(hasattr(r.notes, '__iter__'), True)
        self.assertIs(hasattr(r.platforms, '__iter__'), True)
        self.assertIsInstance(r.references, string_types)
        self.assertIs(type(list(r.references)), list)
        self.assertIsInstance(r.stime, arrow.arrow.Arrow)
        self.assertIsInstance(r.ftime, arrow.arrow.Arrow)
        self.assertIsInstance(r.duration, datetime.timedelta)
        self.assertIsInstance(r.team, string_types)

    def test_package_types(self):
        """
        Test the types returned by Package objects are OK
        """
        p = db.session.query(Package).first()
        x = db.session.query(Package).all()
        self.assertIsInstance(p.id, uuid.UUID)
        self.assertIsInstance(p.name, string_types)
        self.assertIsInstance(p.stime, arrow.arrow.Arrow)
        self.assertIsInstance(p.ftime, arrow.arrow.Arrow)
        self.assertIsInstance(p.duration, datetime.timedelta)
        self.assertIsInstance(p.status, string_types)
        self.assertIsInstance(p.version, string_types)



from __future__ import print_function
import unittest

"""
Test the example deployer
"""


class TestDeployer(unittest.TestCase):
    ENV = {}
    ARGS = {}
    METADATA = {}

    def test_exit_0(self):
        """
        Test the deployer exits 0
        """


from __future__ import print_function
from flask import jsonify, request, Response, json, g
from orlo import app, queries, config
from orlo.exceptions import InvalidUsage
from orlo.user_auth import token_auth
from orlo.orm import db, Release, Package, PackageResult, ReleaseNote, \
    ReleaseMetadata, Platform
from orlo.util import validate_request_json, create_release, \
    validate_release_input, validate_package_input, fetch_release, \
    create_package, fetch_package, stream_json_list, str_to_bool, is_uuid
from orlo.deploy import ShellDeploy
from orlo.user_auth import conditional_auth

__author__ = 'alforbes'


"""
Note - the /packages/ endpoint is very alpha, it is not part of the usual
release workflow and intended for utilitarian uses such as debugging and
fixing errors.
"""


@app.route('/packages', methods=['GET'])
@app.route('/packages/<package_id>', methods=['GET'])
def get_packages(package_id=None):
    """
    Return a list of packages to the client

    :param package_id:
    :return:
    """

    booleans = ('rollback', )

    if package_id:  # Simple, just fetch one package
        if not is_uuid(package_id):
            raise InvalidUsage("Package ID given is not a valid UUID")
        query = queries.get_package(package_id)
    elif len([x for x in request.args.keys()]) == 0:
        raise InvalidUsage("Please specify a filter. See "
                           "http://orlo.readthedocs.org/en/latest/rest.html"
                           "#get--packages for more info")
    else:  # Bit more complex
        # Flatten args, as the ImmutableDict puts some values in a list when
        # expanded
        args = {}
        for k in request.args.keys():
            if k in booleans:
                args[k] = str_to_bool(request.args.get(k))
            else:
                args[k] = request.args.get(k)
        query = queries.packages(**args)

    # Execute eagerly to avoid confusing stack traces within the Response on
    # error
    db.session.execute(query)

    return Response(stream_json_list('packages', query),
                    content_type='application/json')



from __future__ import print_function, unicode_literals
from datetime import datetime, timedelta
import json
import uuid
from orlo.orm import db, Package, Release
from orlo.config import config
from time import sleep
from test_base import OrloTest


class OrloHttpTest(OrloTest):
    """
    Base class for tests which contains the methods required to move
    releases and packages through the workflow via HTTP
    """

    def _create_release(self,
                        user='testuser',
                        team='test team',
                        platforms='test_platform',
                        references='TestTicket-123',
                        ):
        """
        Create a release using the REST API

        :param user:
        :param team:
        :param platforms:
        :param references:
        :return: the release ID
        """

        response = self.client.post(
            '/releases',
            data=json.dumps({
                'note': 'test note lorem ipsum',
                'metadata': {'env': 'test'},
                'platforms': platforms,
                'references': references,
                'team': team,
                'user': user,
            }),
            content_type='application/json',
        )
        self.assert200(response)
        return response.json['id']

    def _create_package(self, release_id,
                        name='test-package',
                        version='1.2.3',
                        diff_url=None,
                        rollback=False,
                        ):
        """
        Create a package using the REST API

        :param release_id: release id to create the package for
        :param name:
        :param version:
        :param diff_url:
        :param rollback:
        :return: package id
        """
        doc = {
            'name': name,
            'version': version,
        }

        if diff_url:
            doc['diff_url'] = diff_url
        if rollback:
            doc['rollback'] = rollback

        response = self.client.post(
            '/releases/{}/packages'.format(release_id),
            data=json.dumps(doc),
            content_type='application/json',
        )
        self.assert200(response)
        return response.json['id']

    def _start_package(self, release_id, package_id):
        """
        Start a package using the REST API

        :param release_id:
        :return:
        """

        response = self.client.post(
            '/releases/{}/packages/{}/start'.format(release_id, package_id),
            content_type='application/json',
        )
        self.assertEqual(response.status_code, 204)
        return response

    def _stop_package(self, release_id, package_id,
                      success=True):
        """
        Start a package using the REST API

        :param release_id:
        :return:
        """

        response = self.client.post(
            '/releases/{}/packages/{}/stop'.format(release_id, package_id),
            data=json.dumps({
                'success': str(success),
                'foo': 'bar',
            }),
            content_type='application/json',
        )
        self.assertEqual(response.status_code, 204)
        return response

    def _stop_release(self, release_id):
        """
        Stop a release using the REST API

        :param release_id:
        :return:
        """
        response = self.client.post(
            '/releases/{}/stop'.format(release_id),
            content_type='application/json',
        )

        self.assertEqual(response.status_code, 204)
        return response

    def _create_finished_release(self):
        """
        Create a completed release, with default values, that has gone through
        the whole workflow
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)

        self._start_package(release_id, package_id)
        self._stop_package(release_id, package_id)
        self._stop_release(release_id)

        return release_id

    def _post_releases_notes(self, release_id, text):
        """
        Add a note to a release
        """

        doc = {'text': text}
        response = self.client.post(
            '/releases/{}/notes'.format(release_id, text),
            data=json.dumps(doc),
            content_type='application/json',
        )
        self.assertEqual(response.status_code, 204)
        return response

    def _post_releases_metadata(self, release_id, metadata):
        """
        Add a metadata to a release
        """

        response = self.client.post(
            '/releases/{}/metadata'.format(release_id, metadata),
            data=json.dumps(metadata),
            content_type='application/json',
        )
        self.assertEqual(response.status_code, 204)
        return response


class BaseUrlTest(OrloHttpTest):
    """
    Test the HTTP GET contract
    """
    def test_ping(self):
        """
        Start with something simple
        """
        response = self.client.get('/ping')
        self.assert200(response)
        self.assertEqual('pong', response.json['message'])

    def test_version(self):
        """
        Test the version url
        """
        from orlo import __version__
        response = self.client.get('/version')
        self.assert200(response)
        self.assertEqual(__version__, response.json['version'])



from __future__ import print_function, unicode_literals
import json
from orlo.orm import Release, Package, Platform, db
from orlo.config import config
from test_route_base import OrloTest

__author__ = 'alforbes'


class TestImport(OrloTest):
    """
    Base import test class

    Common methods and tests that should run against any document.
    Thus we include tests for the essential parameters.
    """

    # Minimal document, override this
    doc = """
    [
      {
        "platforms": [ "GumtreeUK" ],
        "stime": "2015-12-09T12:34:45Z",
        "user": "bob",
        "packages": [
          {
            "name": "test-package-1",
            "version": "0.0.1"
          },
          {
            "name": "test-package-2",
            "version": "0.0.2"
          }
        ]
      }
    ]
    """
    doc_dict = json.loads(doc)

    def setUp(self):
        db.create_all()
        self._import_doc()

    def _import_doc(self):
        """
        Import our test document
        """
        response = self.client.post(
                '/releases/import',
                data=self.doc,
                content_type='application/json',
        )

        self.assert200(response)
        self.release = db.session.query(Release).first()
        self.package = db.session.query(Package).first()
        self.platform = db.session.query(Platform).first()

    def test_import_get_releases(self):
        """
        Test a GET /releases after import

        Crude. If only this test fails, consider adding a more specific test for the cause of
        the failure.
        """
        response = self.client.get('/releases?limit=1')
        self.assert200(response)

    def test_import_param_platforms(self):
        """
        Test that the platforms field is imported successfully
        """
        self.assertEqual(self.platform.name, self.doc_dict[0]['platforms'][0])

    def test_import_param_stime(self):
        """
        Test start time matches
        """
        self.assertEqual(
                self.release.stime.strftime(config.get('main', 'time_format')),
                self.doc_dict[0]['stime'])

    def test_import_param_user(self):
        """
        Test import user matches
        """
        self.assertEqual(self.release.user, self.doc_dict[0]['user'])

    def test_import_param_package_name(self):
        """
        Test package name attributes match
        """
        self.assertEqual(self.package.name, self.doc_dict[0]['packages'][0]['name'])

    def test_import_param_package_version(self):
        """
        Test package version attributes match
        """
        self.assertEqual(self.package.version, self.doc_dict[0]['packages'][0]['version'])


class TestNormalImport(TestImport):
    """
    Normal import test with all fields populated
    """

    doc = """
    [
        {
          "platforms": [ "GumtreeUK" ],
          "ftime": "2015-12-09T12:34:45Z",
          "stime": "2015-12-09T12:34:45Z",
          "team": "Gumtree UK Site Operations",
          "references": [ "GTEPICS-TEST3" ],
          "metadata" :
            {
                "env" : "test1",
                "pool" : "pool1"
            },
          "packages": [
            {
              "status": "SUCCESSFUL",
              "rollback": true,
              "name": "test-package-1",
              "version": "0.0.1",
              "ftime": "2015-12-09T12:34:45Z",
              "stime": "2015-12-09T12:34:45Z",
              "diff_url": "http://example.com/diff"
            },
            {
              "status": "SUCCESSFUL",
              "rollback": true,
              "name": "test-package-2",
              "version": "0.0.2",
              "ftime": "2015-12-09T12:34:45Z",
              "stime": "2015-12-09T12:34:45Z",
              "diff_url": null
            }
          ],
          "user": "bob",
          "notes": [
            "note 1",
            "note 2"
          ]
        }
    ]
    """
    doc_dict = json.loads(doc)

    def test_import_param_ftime(self):
        """
        Test finish time matches
        """

        self.assertEqual(
            self.release.ftime.strftime(config.get('main', 'time_format')),
            self.doc_dict[0]['stime'])

    def test_import_param_team(self):
        """
        Test imported team matches
        """
        self.assertEqual(self.release.team, self.doc_dict[0]['team'])

    def test_import_param_references(self):
        """
        Test imported references match
        """
        doc = json.loads(self.release.references)
        self.assertEqual(doc, self.doc_dict[0]['references'])

    def test_import_param_references_valid_json(self):
        """
        Test the reference field results in valid json
        """
        doc = json.loads(self.release.references)
        self.assertIsInstance(doc, list)

    def test_import_param_package_status(self):
        """
        Test package status attributes match
        """
        self.assertEqual(self.package.status, self.doc_dict[0]['packages'][0]['status'])

    def test_import_param_package_diff_url(self):
        """
        Test package diff_url is recorded
        """
        self.assertEqual(self.package.diff_url, self.doc_dict[0]['packages'][0]['diff_url'])

    def test_import_param_package_ftime(self):
        """
        Test package ftime attributes match
        """
        self.assertEqual(
            self.package.ftime.strftime(config.get('main', 'time_format')),
            self.doc_dict[0]['packages'][0]['ftime'])

    def test_import_param_package_stime(self):
        """
        Test package stime attributes match
        """
        self.assertEqual(
            self.package.stime.strftime(config.get('main', 'time_format')),
            self.doc_dict[0]['packages'][0]['stime'])

    def test_import_param_notes(self):
        """
        Test that notes are imported successfully
        """
        for note in self.release.notes:
            self.assertIn(note.content, self.doc_dict[0]['notes'])


class TestNullTime(TestImport):
    def test_import_release_ftime_null(self):
        """
        Release ftime should be null (None) for the minimal document
        """
        self.assertIs(self.release.ftime, None)

    def test_import_package_ftime_null(self):
        """
        Package ftime should be None for the minimal document
        """
        self.assertIs(self.package.ftime, None)

    def test_import_package_stime_equals_release(self):
        """
        We use the release's stime for packages that don't define it
        """
        self.assertEqual(
            self.package.stime.strftime(config.get('main', 'time_format')),
            self.doc_dict[0]['stime'])




from __future__ import print_function
from flask_testing import TestCase
from flask import jsonify
import orlo
from orlo.orm import db
from orlo.user_auth import user_auth, token_auth, conditional_auth
from test_base import ConfigChange
from werkzeug.datastructures import Headers
from werkzeug.test import Client
import base64
import ldap
from mockldap import MockLdap
import sys
if sys.version_info[0] < 3:
    from mock import patch
else:
    from unittest.mock import patch
from orlo.config import config

__author__ = 'alforbes'

USERNAME = 'testuser'
PASSWORD = 'blah'


config.set('security', 'passwd_file', '')


# A couple of test routes which require auth, and return 200 if it succeeds
@orlo.app.route('/test/token_required')
@conditional_auth(token_auth.token_required)
def token_required():
    response = jsonify({'message': 'OK'})
    response.status_code = 200
    return response


@orlo.app.route('/test/auth_required')
@conditional_auth(user_auth.login_required)
def auth_required():
    response = jsonify({'message': 'OK'})
    response.status_code = 200
    return response


@orlo.app.route('/test/user')
@conditional_auth(user_auth.login_required)
def get_resource():
    return jsonify({'data': 'Hello, %s!' % g.current_user})


class OrloAuthTest(TestCase):
    """
    Base test class to setup the app
    """
    top = ('o=test', {'o': ['test']})
    example = ('ou=example,o=test', {'ou': ['example']})
    people = ('ou=people,ou=example,o=test', {'ou': ['other']})
    ldapuser = ('uid=ldapuser,ou=people,ou=example,o=test',
                {'uid': ['ldapuser'], 'userPassword': ['ldapuserpw']})
    # This is the content of our mock LDAP directory. It takes the form
    # {dn: {attr: [value, ...], ...}, ...}.
    directory = dict([top, example, people, ldapuser])

    def create_app(self):
        self.app = orlo.app
        self.app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite://'
        self.app.config['TESTING'] = True
        self.app.config['DEBUG'] = False
        self.app.config['TRAP_HTTP_EXCEPTIONS'] = True
        self.app.config['PRESERVE_CONTEXT_ON_EXCEPTION'] = False

        return self.app

    @classmethod
    def setUpClass(self):
        # We only need to create the MockLdap instance once. The content we
        # pass in will be used for all LDAP connections.
        self.mockldap = MockLdap(self.directory)

    @classmethod
    def tearDownClass(self):
        del self.mockldap

    def setUp(self):
        db.create_all()
        self.mockldap.start()
        self.ldapobj = self.mockldap['ldap://localhost/']
        self.orig_security_enabled = orlo.config.get('security', 'enabled')
        self.orig_security_secret_key = orlo.config.set('security',
                                                        'secret_key')
        self.orig_security_ldap_server = orlo.config.set('security',
                                                         'ldap_server')
        self.orig_security_ldap_port = orlo.config.set('security', 'ldap_port')
        self.orig_security_user_base_dn = orlo.config.set('security',
                                                          'user_base_dn')
        orlo.config.set('security', 'enabled', 'true')
        orlo.config.set('security', 'secret_key', 'It does not matter how '
                                                  'slowly you go so long as '
                                                  'you do not stop')
        orlo.config.set('security', 'ldap_server', 'localhost')
        orlo.config.set('security', 'ldap_port', '389')
        orlo.config.set('security', 'user_base_dn',
                        'ou=people,ou=example,o=test')

    def tearDown(self):
        db.session.remove()
        db.drop_all()
        self.mockldap.stop()
        del self.ldapobj
        orlo.config.set('security', 'enabled', self.orig_security_enabled)
        orlo.config.set('security', 'secret_key', self.orig_security_secret_key)

    def get_with_basic_auth(self, path, username='testuser', password='blah'):
        """
        Do a request with ldap auth

        :param path:
        :param username:
        :param password:
        """
        h = Headers()
        s_auth = base64.b64encode('{u}:{p}'.format(
            u=username, p=password).encode('utf-8'))
        h.add('Authorization', 'Basic ' + s_auth.decode('utf-8'))
        response = Client.open(self.client, path=path, headers=h)
        return response

    def get_with_token_auth(self, path, token):
        """
        Do a request with token auth

        :param token:
        :param path:
        """
        h = Headers()
        h.add('X-Auth-Token', token)
        response = Client.open(self.client, path=path, headers=h)
        return response

    def post_with_token_auth(self, path, token, data):
        """
        Do a request with token auth

        :param token:
        :param path:
        """
        h = Headers()
        h.add('X-Auth-Token', token)
        h.add('Content-Type', 'application/json')
        response = Client.open(self.client, method='POST', data=data, path=path,
                               headers=h)
        return response

    def get_token(self):
        response = self.get_with_basic_auth('/token')
        return response.json['token']


@patch('orlo.user_auth.verify_password_file')
class TestOrloAuthToken(OrloAuthTest):
    def test_fetch_token(self, mock_verify_password_file):
        """
        Test that we can fetch a token
        """
        mock_verify_password_file.return_value = True
        response = self.get_with_basic_auth('/token')
        self.assert200(response)
        self.assertIn('token', response.json)


@patch('orlo.user_auth.verify_password_file')
class TestTokenAuth(OrloAuthTest):
    """
    Parent class for testing Token authentication
    """
    URL_PATH = '/test/token_required'

    def test_unauthorised(self, mock_verify_password):
        """
        Test that we get 401 without a token
        """
        response = self.client.get(self.URL_PATH)
        self.assert401(response)

    def test_auth_disabled(self, mock_verify_password):
        """
        Same test as above with auth disabled
        """
        with ConfigChange('security', 'enabled', 'false'):
            response = self.client.get(self.URL_PATH)
            self.assert200(response)

    def test_with_token(self, mock_verify_password):
        token = self.get_token()
        response = self.get_with_token_auth(self.URL_PATH, token)
        self.assert200(response)

    def test_with_bad_token(self, mock_verify_password):
        token = 'foo'
        response = self.get_with_token_auth(self.URL_PATH, token)
        self.assert401(response)


@patch('orlo.user_auth.verify_password_file')
class TestUserAuth(OrloAuthTest):
    """
    Parent class for testing HTTP Basic authentication
    """
    URL_PATH = '/test/auth_required'

    def test_auth_disabled(self, mock_verify_password):
        """
        Same test as above with auth disabled
        """
        with ConfigChange('security', 'enabled', 'false'):
            response = self.client.get(self.URL_PATH)
            self.assert200(response)

    def test_with_login(self, mock_verify_password):
        response = self.get_with_basic_auth('/test/auth_required')
        self.assert200(response)

    def test_with_ldap_login(self, mock_verify_password):
        response = self.get_with_basic_auth(
            '/test/auth_required', username='ldapuser', password='ldapuserpw'
        )
        self.assert200(response)

    def test_with_bad_login(self, mock_verify_password):
        mock_verify_password.return_value = False
        response = self.get_with_basic_auth(
            '/test/auth_required', username='bad_user', password='foobar'
        )
        self.assert401(response)


class TestReleasesAuth(OrloAuthTest):
    """
    Test auth against a real url, /releases

    Bit more complicated as we have to do some POST requests
    """
    URL_PATH = '/releases'

    def test_get_returns_200(self):
        """
        No auth required for GETs
        """
        url = self.URL_PATH + '?platform=foo'
        response = self.client.get(url)
        self.assert200(response)

    def test_post_releases_returns_401(self):
        """
        Test auth fails
        """
        response = self.client.post(self.URL_PATH, data={'foo': 'bar'})
        self.assert401(response)

    @patch('orlo.user_auth.verify_password_file')
    def test_post_releases_with_token_returns_400(
            self, mock_verify_password_file):
        """
        Test auth succeeds
        """
        token = self.get_token()
        response = self.post_with_token_auth(
            self.URL_PATH, token=token, data={'foo': 'bar'},
        )
        self.assert400(response)
        self.assertIn(b'message', response.data)



from __future__ import print_function
from test_route_base import OrloHttpTest

__author__ = 'alforbes'


class TestInfoUrl(OrloHttpTest):
    def test_info_root(self):
        """
        Test /info returns 200
        """
        response = self.client.get('/info')
        self.assert200(response)

    def test_info_users(self):
        """
        Test /info/users returns 200
        """
        self._create_release(user='userOne')
        response = self.client.get('/info/users')
        self.assert200(response)
        self.assertIn('userOne', response.json)

    def test_info_users_with_user(self):
        """
        Test /info/users/username returns 200
        """
        self._create_release(user='userOne')
        response = self.client.get('/info/users/userOne')
        self.assert200(response)
        self.assertIn('userOne', response.json)

    def test_info_users_with_platform(self):
        """
        Test /info?platform=<platform> returns 200
        """
        self._create_release(user='userOne', platforms=['platformOne'])
        response = self.client.get('/info/users?platform=platformOne')
        self.assert200(response)
        self.assertIn('userOne', response.json)

    def test_info_platforms(self):
        """
        Test /info/platforms returns 200
        """
        self._create_release(platforms=['platformOne'])
        response = self.client.get('/info/platforms')
        self.assert200(response)
        self.assertIn('platformOne', response.json)

    def test_info_platforms_with_platform(self):
        """
        Test /info/platforms/<platform> returns 200
        """
        self._create_release(platforms=['platformOne'])
        response = self.client.get('/info/platforms/platformOne')
        self.assert200(response)
        self.assertIn('platformOne', response.json)

    def test_info_platforms_with_platform_negative(self):
        """
        Test /info/platforms/<platform> returns 200
        """
        self._create_release(platforms=['platformOne'])
        response = self.client.get('/info/platforms/badPlatform')
        self.assert200(response)
        self.assertNotIn('platformOne', response.json)

    def test_info_packages(self):
        """
        Test /info/packages
        """

        self._create_finished_release()
        response = self.client.get('/info/packages')
        self.assert200(response)
        self.assertIn('test-package', response.json)

    def test_info_packages_with_package(self):
        """
        Test /info/package with a package
        """
        rid = self._create_release()
        self._create_package(rid, name='packageOne')
        response = self.client.get('/info/packages/packageOne')
        self.assert200(response)
        self.assertIn('packageOne', response.json)

    def test_info_packages_with_platform(self):
        """
        Test /info/package with a platform filter
        """
        rid = self._create_release(platforms=['platformOne'])
        self._create_package(rid, name='packageOne')
        response = self.client.get('/info/packages?platform=platformOne')
        self.assert200(response)
        self.assertIn('packageOne', response.json)

    def test_info_packages_with_platform_negative(self):
        """
        Test /info/package with a platform filter
        """
        rid = self._create_release(platforms=['platformOne'])
        self._create_package(rid, name='packageOne')
        response = self.client.get('/info/packages?platform=platformFoo')
        self.assert200(response)
        self.assertNotIn('packageOne', response.json)

    def test_info_packages_list(self):
        """
        Test /info/package_list
        """

        self._create_finished_release()
        response = self.client.get('/info/packages/list')
        self.assert200(response)
        self.assertIn('test-package', response.json['packages'])

    def test_info_packages_list_with_platform(self):
        """
        Test /info/package_list
        """

        self._create_finished_release()
        response = self.client.get('/info/packages/list?platform=test_platform')
        self.assert200(response)
        self.assertIn('test-package', response.json['packages'])

    def test_info_packages_list_with_platform_negative(self):
        """
        Test /info/package_list
        """

        self._create_finished_release()
        response = self.client.get('/info/packages/list?platform=non-existent-platform')
        self.assert200(response)
        self.assertNotIn('test-package', response.json['packages'])

    def test_info_packages_versions(self):
        """
        Test /info/packages returns 200
        """
        self._create_finished_release()
        response = self.client.get('/info/packages/versions')
        self.assert200(response)
        self.assertIn('test-package', response.json)

    def test_info_packages_versions_with_platform(self):
        """
        Test /info/packages returns 200
        """
        self._create_finished_release()
        response = self.client.get('/info/packages/versions?platform=test_platform')
        self.assert200(response)
        self.assertIn('test-package', response.json)

    def test_info_package_versions_with_platform_negative(self):
        """
        Test /info/packages returns 200
        """
        self._create_finished_release()
        response = self.client.get('/info/packages/versions?platform=non-existent-platform')
        self.assert200(response)
        self.assertNotIn('test-package', response.json)



from __future__ import print_function, unicode_literals
from datetime import datetime, timedelta
import json
import uuid
from orlo.orm import db, Package, Release
from orlo.config import config
from time import sleep
from test_route_base import OrloHttpTest
from test_base import OrloLiveTest


__author__ = 'alforbes'


class TestPostContract(OrloHttpTest):
    """
    Test the HTTP POST contract
    """

    def test_create_release(self):
        """
        Create a release
        """
        release_id = self._create_release()
        self.assertEqual(uuid.UUID(release_id).version, 4)

    def test_create_package(self):
        """
        Create a package
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        self.assertEqual(uuid.UUID(package_id).version, 4)

    def test_add_results(self):
        """
        Add the results of a package deploy
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)

        results_response = self.client.post(
            '/releases/{}/packages/{}/results'.format(
                release_id, package_id),
            data=json.dumps({
                'success': 'true',
                'foo': 'bar',
            }),
            content_type='application/json',
        )
        self.assertEqual(results_response.status_code, 204)

    def test_package_start(self):
        """
        Test starting a package
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)

        return release_id, package_id

    def test_package_stop(self):
        """
        Test stopping a package
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        self._start_package(release_id, package_id)
        self._stop_package(release_id, package_id)

    def test_release_stop(self):
        """
        Test stopping a release

        As before, calls all the others as we need a full workflow present
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        self._start_package(release_id, package_id)
        self._stop_package(release_id, package_id)
        self._stop_release(release_id)

    def test_post_note(self):
        """
        Test adding a release
        """
        release_id = self._create_release()
        response = self._post_releases_notes(release_id, "this is a test message")
        self.assertEqual(204, response.status_code)

    def test_post_metadata(self):
        """
        Test adding metadata to a release
        """
        release_id = self._create_release()
        response = self._post_releases_metadata(release_id, {"meta": "data"})
        self.assertEqual(204, response.status_code)

    def test_create_release_minimal(self):
        """
        Create a release, omitting all optional parameters
        """
        response = self.client.post('/releases',
                                    data=json.dumps({
                                        'platforms': ['test_platform'],
                                        'user': 'testuser',
                                    }),
                                    content_type='application/json',
                                    )
        self.assert200(response)

    def test_diffurl_present(self):
        """
        Test that the diff_url parameter is stored
        """
        test_url = 'http://example.com'

        release_id = self._create_release()
        package_id = self._create_package(release_id, diff_url=test_url)

        q = db.session.query(Package).filter(Package.id == package_id)
        pkg = q.first()
        self.assertEqual(pkg.diff_url, test_url)

    def test_rollback_present(self):
        """
        Test that that rollback parameter is stored
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id, rollback=True)

        q = db.session.query(Package).filter(Package.id == package_id)
        pkg = q.first()

        self.assertEqual(pkg.rollback, True)

    def test_references_json_conversion(self):
        """
        Test that the references parameter results in valid json in the database
        """
        release_id = self._create_release(references=['ticket1', 'ticket2'])
        q = db.session.query(Release).filter(Release.id == release_id)
        release = q.first()

        doc = json.loads(release.references)
        self.assertIsInstance(doc, list)

    def test_stop_package_success_true(self):
        """
        Test stop_package when success is false
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        self._start_package(release_id, package_id)
        self._stop_package(release_id, package_id, success=True)

        release = db.session.query(Release).filter(Release.id == release_id).one()
        package = release.packages[0]
        self.assertEqual(package.status, 'SUCCESSFUL')

    def test_stop_package_success_false(self):
        """
        Test stop_package when success is false
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        self._start_package(release_id, package_id)
        self._stop_package(release_id, package_id, success=False)

        release = db.session.query(Release).filter(Release.id == release_id).one()
        package = release.packages[0]
        self.assertEqual(package.status, 'FAILED')


class TestGetContract(OrloHttpTest):
    """
    Test the HTTP GET contract
    """

    def _get_releases(self, release_id=None, filters=None, expected_status=200):
        """
        Perform a GET to /releases with optional filters
        """

        if release_id:
            path = '/releases/{}'.format(release_id)
        elif filters:
            path = '/releases?{}'.format('&'.join(filters))
        else:
            path = '/releases'

        results_response = self.client.get(
            path, content_type='application/json',
        )

        try:
            self.assertEqual(results_response.status_code, expected_status)
        except AssertionError as err:
            print(results_response.data)
            raise
        r_json = json.loads(results_response.data.decode('utf-8'))
        return r_json

    def test_get_single_release(self):
        """
        Fetch a single release
        """
        release_id = self._create_release()
        r = self._get_releases(release_id=release_id)

        self.assertEqual(1, len(r['releases']))
        self.assertEqual(release_id, r['releases'][0]['id'])

    def test_get_single_release_invalid(self):
        """
        Test that we return 400 with an invalid release ID
        """
        r = self._get_releases(release_id="not_a_valid_uuid",
                               expected_status=400)

    def test_get_releases(self):
        """
        Test the list of releases
        """
        for _ in range(0, 3):
            self._create_finished_release()
        results = self._get_releases(
            filters=['limit=10']
        )
        self.assertEqual(len(results['releases']), 3)

    def test_get_release_filter_package(self):
        """
        Filter on releases that contain a package
        """
        for _ in range(0, 3):
            self._create_finished_release()

        release_id = self._create_release()
        package_id = self._create_package(release_id, name='specific-package')
        results = self._get_releases(filters=[
            'package_name=specific-package'
        ])

        for r in results['releases']:
            for p in r['packages']:
                self.assertEqual(p['name'], 'specific-package')
                self.assertEqual(p['id'], package_id)

        self.assertEqual(len(results['releases']), 1)

    def test_get_release_filter_user(self):
        """
        Filter on releases that were performed by a user
        """
        for _ in range(0, 3):
            self._create_release(user='firstUser')

        for _ in range(0, 2):
            self._create_release(user='secondUser')

        first_results = self._get_releases(filters=['user=firstUser'])
        second_results = self._get_releases(filters=['user=secondUser'])

        self.assertEqual(len(first_results['releases']), 3)
        self.assertEqual(len(second_results['releases']), 2)

        for r in first_results['releases']:
            self.assertEqual(r['user'], 'firstUser')
        for r in second_results['releases']:
            self.assertEqual(r['user'], 'secondUser')

    def test_get_release_filter_platform(self):
        """
        Filter on releases that were on a particular platform
        """
        for _ in range(0, 3):
            self._create_release(platforms='firstPlatform')

        for _ in range(0, 2):
            self._create_release(platforms=['firstPlatform', 'secondPlatform'])

        first_results = self._get_releases(filters=['platform=firstPlatform'])
        second_results = self._get_releases(filters=['platform=secondPlatform'])

        # Should be all releases
        self.assertEqual(len(first_results['releases']), 5)
        # Just those that contain "secondPlatform"
        self.assertEqual(len(second_results['releases']), 2)

        for r in first_results['releases']:
            self.assertIn('firstPlatform', r['platforms'])
        for r in second_results['releases']:
            self.assertIn('secondPlatform', r['platforms'])

    def _get_releases_time_filter(self, field, finished=False):
        """
        Return releases given the time-based filter field

        Abstracts a bit of common code in the stime/ftime tests
        """
        for _ in range(0, 3):
            if finished:
                self._create_finished_release()
            else:
                self._create_release()

        t_format = config.get('main', 'time_format')
        now = datetime.utcnow()

        yesterday = (now - timedelta(days=1)).strftime(t_format)
        tomorrow = (now + timedelta(days=1)).strftime(t_format)

        r_yesterday = self._get_releases(
            filters=['{}={}'.format(field, yesterday)]
        )
        r_tomorrow = self._get_releases(
            filters=['{}={}'.format(field, tomorrow)]
        )

        return r_yesterday, r_tomorrow

    def test_get_release_filter_stime_before(self):
        """
        Filter on releases that started before a particular time
        """
        r_yesterday, r_tomorrow = self._get_releases_time_filter('stime_before')

        self.assertEqual(3, len(r_tomorrow['releases']))
        self.assertEqual(0, len(r_yesterday['releases']))

    def test_get_release_filter_ftime_null(self):
        """
        Filter on releases that don't have a finish time
        """
        releases = self._get_releases(
            filters=['{}={}'.format('ftime', 'null')]
        )
        self.assertEqual(len(releases['releases']), 0)

    def test_get_release_filter_stime_after(self):
        """
        Filter on releases that started after a particular time
        """
        r_yesterday, r_tomorrow = self._get_releases_time_filter('stime_after')

        self.assertEqual(0, len(r_tomorrow['releases']))
        self.assertEqual(3, len(r_yesterday['releases']))

    def test_get_release_filter_ftime_before(self):
        """
        Filter on releases that finished before a particular time
        """
        r_yesterday, r_tomorrow = self._get_releases_time_filter(
            'ftime_before', finished=True)

        self.assertEqual(3, len(r_tomorrow['releases']))
        self.assertEqual(0, len(r_yesterday['releases']))

    def test_get_release_filter_ftime_after(self):
        """
        Filter on releases that finished after a particular time
        """
        r_yesterday, r_tomorrow = self._get_releases_time_filter(
            'ftime_after', finished=True)

        self.assertEqual(0, len(r_tomorrow['releases']))
        self.assertEqual(3, len(r_yesterday['releases']))

    def test_get_release_filter_duration_lt(self):
        """
        Filter on releases that took less than x seconds

        All releases take a very small amount of time in our tests,
        but all should be <10 and >0 at least!
        """
        for _ in range(0, 3):
            self._create_finished_release()

        r = self._get_releases(filters=['duration_lt=10'])
        self.assertEqual(3, len(r['releases']))

        r = self._get_releases(filters=['duration_lt=0'])
        self.assertEqual(0, len(r['releases']))

    def test_get_release_filter_duration_gt(self):
        """
        Filter on releases that took greater than x seconds
        """
        for _ in range(0, 3):
            self._create_finished_release()

        r = self._get_releases(filters=['duration_gt=10'])
        self.assertEqual(0, len(r['releases']))

        r = self._get_releases(filters=['duration_gt=0'])
        self.assertEqual(3, len(r['releases']))

    def test_get_release_filter_team(self):
        """
        Filter on releases that a team was responsible for
        """
        for _ in range(0, 3):
            self._create_release(team='firstTeam')

        for _ in range(0, 2):
            self._create_release(team='secondTeam')

        first_results = self._get_releases(filters=['team=firstTeam'])
        second_results = self._get_releases(filters=['team=secondTeam'])

        self.assertEqual(len(first_results['releases']), 3)
        self.assertEqual(len(second_results['releases']), 2)

        for r in first_results['releases']:
            self.assertEqual(r['team'], 'firstTeam')
        for r in second_results['releases']:
            self.assertEqual(r['team'], 'secondTeam')

    def test_get_release_filter_rollback(self):
        """
        Filter on releases that contain a rollback
        """
        for _ in range(0, 3):
            rid = self._create_release()
            self._create_package(rid, rollback=True)

        for _ in range(0, 2):
            rid = self._create_release()
            self._create_package(rid, rollback=False)

        first_results = self._get_releases(filters=['package_rollback=True'])
        second_results = self._get_releases(filters=['package_rollback=False'])

        for r in first_results['releases']:
            for p in r['packages']:
                self.assertIs(p['rollback'], True)
        for r in second_results['releases']:
            for p in r['packages']:
                self.assertIs(p['rollback'], False)

        self.assertEqual(len(first_results['releases']), 3)
        self.assertEqual(len(second_results['releases']), 2)

    def test_get_release_filter_reference(self):
        """
        Test filtering on reference
        """
        for _ in range(0, 3):
            rid = self._create_release(references='REF')
            self._create_package(rid)
        for _ in range(0, 2):
            rid = self._create_release()
            self._create_package(rid)

        first_results = self._get_releases(filters=['reference=REF'])
        second_results = self._get_releases(filters=['reference=ZERO'])

        self.assertEqual(len(first_results['releases']), 3)
        self.assertEqual(len(second_results['releases']), 0)

        for r in first_results['releases']:
            self.assertEqual(r['references'], ['REF'])

    def test_get_release_limit_one(self):
        """
        Should return only one release
        """

        for _ in range(0, 3):
            self._create_release()
            sleep(0.1)

        r = self._get_releases(filters=['limit=1'])
        self.assertEqual(len(r['releases']), 1)

    def test_get_release_offset(self):
        """
        Test that offset=1 skips the first release
        """
        rid = None
        for _ in range(0, 2):
            rid = self._create_release()
            sleep(0.1)

        r = self._get_releases(filters=['offset=1'])
        self.assertEqual(len(r['releases']), 1)
        self.assertEqual(r['releases'][0]['id'], rid)

    def test_get_release_desc(self):
        """
        Should return in reverse order
        """

        rid = None
        for _ in range(0, 3):
            rid = self._create_release()
            sleep(0.1)

        r = self._get_releases(filters=['desc=true'])
        # First in list should be last to be created
        self.assertEqual(r['releases'][0]['id'], rid)

    def test_get_release_package_name(self):
        """
        Filter on releases which have a particular package name
        """
        rid = self._create_release()
        self._create_package(rid, name='particular-name')
        self._create_package(rid, name='another-name')

        rid2 = self._create_release()
        self._create_package(rid2, name='another-name')

        r = self._get_releases(filters=['package_name=particular-name'])
        self.assertEqual(len(r['releases']), 1)

    def test_get_release_package_version(self):
        """
        Filter on releases which have a particular version
        """
        rid = self._create_release()
        self._create_package(rid, version='4.9.9')
        self._create_package(rid, version='3.5.6')

        rid2 = self._create_release()
        self._create_package(rid2, version='1.2.3')

        r = self._get_releases(filters=['package_version=4.9.9'])
        self.assertEqual(len(r['releases']), 1)

    def test_get_release_package_status_not_started(self):
        """
        Filter on releases which have package status NOT_STARTED
        """
        rid = self._create_release()
        self._create_package(rid)

        self._create_release()  # extra, should be filtered out

        r = self._get_releases(filters=['package_status=NOT_STARTED'])
        self.assertEqual(len(r['releases']), 1)

    def test_get_release_package_status_in_progress(self):
        """
        Filter on releases which have package status IN_PROGRESS
        """
        rid = self._create_release()
        pid = self._create_package(rid)
        self._start_package(rid, pid)

        self._create_release()  # extra, should be filtered out

        r = self._get_releases(filters=['package_status=IN_PROGRESS'])
        self.assertEqual(len(r['releases']), 1)

    def test_get_release_package_status_successful(self):
        """
        Filter on releases which have package status SUCCESSFUL
        """
        rid = self._create_release()
        pid = self._create_package(rid)
        self._start_package(rid, pid)
        self._stop_package(rid, pid, success=True)

        self._create_release()  # extra, should be filtered out

        r = self._get_releases(filters=['package_status=SUCCESSFUL'])
        self.assertEqual(len(r['releases']), 1)

    def test_get_release_package_status_failed(self):
        """
        Filter on releases which have package status FAILED
        """
        rid = self._create_release()
        pid = self._create_package(rid)
        self._start_package(rid, pid)
        self._stop_package(rid, pid, success=False)

        self._create_release()  # extra, should be filtered out

        r = self._get_releases(filters=['package_status=FAILED'])
        self.assertEqual(len(r['releases']), 1)

    def test_get_release_package_duration_gt(self):
        """
        Filter on releases with a package of duration greater than X
        """
        rid = self._create_release()
        pid = self._create_package(rid)
        self._start_package(rid, pid)
        self._stop_package(rid, pid, success=False)

        r = self._get_releases(filters=['package_duration_gt=0'])
        self.assertEqual(len(r['releases']), 1)

    def test_get_release_package_duration_lt(self):
        """
        Filter on releases with a package of duration less than X
        """
        rid = self._create_release()
        pid = self._create_package(rid)
        self._start_package(rid, pid)
        self._stop_package(rid, pid, success=False)

        r = self._get_releases(filters=['package_duration_lt=10'])
        self.assertEqual(len(r['releases']), 1)

        # TODO need control release

    def test_get_release_filter_rollback_and_status(self):
        """
        Filter on rollback and status
        """
        for _ in range(0, 3):
            rid = self._create_release()
            self._create_package(rid, rollback=True)

        for _ in range(0, 2):
            rid = self._create_release()
            self._create_package(rid, rollback=False)

        first_results = self._get_releases(filters=['package_rollback=True',
                                                    'package_status=NOT_STARTED'])
        second_results = self._get_releases(filters=['package_rollback=False',
                                                     'package_status=NOT_STARTED'])
        # should be zero
        third_results = self._get_releases(filters=['package_rollback=False',
                                                    'package_status=SUCCESSFUL'])

        self.assertEqual(len(first_results['releases']), 3)
        self.assertEqual(len(second_results['releases']), 2)
        self.assertEqual(len(third_results['releases']), 0)

        for r in first_results['releases']:
            for p in r['packages']:
                self.assertIs(p['rollback'], True)
        for r in second_results['releases']:
            for p in r['packages']:
                self.assertIs(p['rollback'], False)

    def test_get_release_bad_attribute(self):
        """
        Test that we get the appropriate status code and a message when sending a bad attribute
        """

        r = self._get_releases(filters=['foo=bar'], expected_status=400)
        self.assertIn('message', r)

    def test_get_release_with_status_successful(self):
        """
        Test that the release status filters correctly
        """
        # A partially successful release (should be considered "FAILED")
        rid1 = self._create_release()
        pid1 = self._create_package(rid1, name='successful_package')
        self._start_package(rid1, pid1)
        self._stop_package(rid1, pid1, success=True)
        pid2 = self._create_package(rid1, name='failed_package')
        self._start_package(rid1, pid2)
        self._stop_package(rid1, pid2, success=False)
        self._stop_release(rid1)

        for _ in range(0, 2):
            # These should be successful
            self._create_finished_release()

        success_results = self._get_releases(filters=['status=SUCCESSFUL'])
        self.assertEqual(len(success_results['releases']), 2)

        failed_results = self._get_releases(filters=['status=FAILED'])
        self.assertEqual(len(failed_results['releases']), 1)
        self.assertEqual(rid1, failed_results['releases'][0]['id'])

    def test_get_release_with_bad_status(self):
        """
        Tests get /releases?status=garbage give a helpful message
        """
        self._create_finished_release()

        result = self._get_releases(filters=['status=garbage_boz'], expected_status=400)
        self.assertIn('message', result)

    def test_get_releases_with_no_filters(self):
        """
        Test get /releases without filters returns 400
        """
        self._get_releases(expected_status=400)





import orlo
import unittest
import os
from flask_testing import TestCase, LiveServerTestCase
from orlo.orm import db
from orlo.orm import Release, db, Package
from orlo.util import append_or_create_platforms

try:
    TRAVIS = True if os.environ['TRAVIS'] == 'true' else False
except KeyError:
    TRAVIS = False


class OrloTest(TestCase):
    """
    Base test class to setup the app
    """
    def create_app(self):
        if TRAVIS:
            db_uri = 'postgres://orlo:password@localhost:5432/orlo'
        else:
            db_uri = 'postgres://orlo:password@192.168.57.100:5432/orlo'
        app = orlo.app
        app.config['SQLALCHEMY_DATABASE_URI'] = db_uri
        # app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite://'
        app.config['TESTING'] = True
        app.config['DEBUG'] = False
        app.config['TRAP_HTTP_EXCEPTIONS'] = True
        app.config['PRESERVE_CONTEXT_ON_EXCEPTION'] = False

        return orlo.app

    def setUp(self):
        db.create_all()
        db.session.begin_nested()

    def tearDown(self):
        db.session.rollback()
        db.session.remove()
        db.drop_all()
        db.engine.dispose()


class OrloLiveTest(LiveServerTestCase):
    """
    Test on a live server
    """

    def create_app(self):
        if TRAVIS:
            db_uri = 'postgres://orlo:password@localhost:5432/orlo'
        else:
            db_uri = 'postgres://orlo:password@192.168.57.100:5432/orlo'

        app = orlo.app
        app.config['SQLALCHEMY_DATABASE_URI'] = db_uri
        # app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite://'
        app.config['TESTING'] = True
        app.config['DEBUG'] = False
        app.config['TRAP_HTTP_EXCEPTIONS'] = True
        app.config['PRESERVE_CONTEXT_ON_EXCEPTION'] = False
        app.config['LIVESERVER_PORT'] = 8943
        app.config['LIVESERVER_TIMEOUT'] = 5

        return orlo.app

    def setUp(self):
        # db.engine.dispose()
        db.create_all()
        db.session.begin_nested()

    def tearDown(self):
        db.session.rollback()
        db.session.close()
        db.drop_all()  # stall here implies connection leak
        db.session.remove()
        db.get_engine(self.app).dispose()


class ReleaseDbUtil(object):
    """
    Utility functions for creating releases in the database
    """
    @classmethod
    def _create_release(cls,
                        user='testuser',
                        team='test team',
                        platforms=None,
                        references=None,
                        success=True):
        """
        Create a release using internal methods

        :param user:
        :param team:
        :param platforms:
        :param references:
        :return:
        """

        if not platforms:
            platforms = ['test_platform']
        if type(platforms) is not list:
            raise AssertionError("Platforms parameter must be list")
        if not references:
            references = ['TestTicket-123']

        db_platforms = append_or_create_platforms(platforms)

        r = Release(
            platforms=db_platforms,
            user=user,
            references=references,
            team=team,
        )
        db.session.add(r)
        db.session.commit()

        return r.id

    @classmethod
    def _create_package(cls,
                        release_id,
                        name='test-package',
                        version='1.2.3',
                        diff_url=None,
                        rollback=False,
                        ):
        """
        Create a package using internal methods

        :param release_id:
        :param name:
        :param version:
        :param diff_url:
        :param rollback:
        :return:
        """
        p = Package(
            release_id=release_id,
            name=name,
            version=version,
            diff_url=diff_url,
            rollback=rollback,
        )
        db.session.add(p)
        db.session.commit()

        return p.id

    @classmethod
    def _start_package(cls, package_id):
        """
        Start a package

        :param package_id:
        """
        package = db.session.query(Package).filter(Package.id == package_id).one()
        package.start()
        db.session.commit()

        return package.id

    @classmethod
    def _stop_package(cls, package_id, success=True):
        """
        Stop a package

        :param package_id:
        """
        package = db.session.query(Package).filter(Package.id == package_id).one()
        package.stop(success=success)
        db.session.commit()

    @classmethod
    def _stop_release(cls, release_id):
        """
        Stop a release

        :param release_id:
        """
        release = db.session.query(Release).filter(Release.id == release_id).one()
        release.stop()
        db.session.commit()

    @classmethod
    def _create_finished_release(cls, success=True):
        """
        Create a completed release using internal methods
        """

        release_id = cls._create_release()
        package_id = cls._create_package(release_id)

        cls._start_package(package_id)
        cls._stop_package(package_id, success=success)
        cls._stop_release(release_id)
        db.session.commit()


class ConfigChange(object):
    def __init__(self, section, option, value):
        """
        Decorator to temporarily change configuration

        @param section: ConfigParser section
        @param option: ConfigParser option
        @param value: Value to set
        @return:
        """
        self.section = section
        self.option = option
        self.value = value

    def __enter__(self):
        self.original_state = orlo.config.get(self.section, self.option)
        orlo.config.set(self.section, self.option, self.value)

    def __exit__(self, exc_type, exc_val, exc_tb):
        orlo.config.set(self.section, self.option, self.original_state)



from __future__ import print_function

__author__ = 'alforbes'

from orlo import app
from orlo.orm import db

app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///orlo.db'
app.debug = True
db.create_all()





from __future__ import print_function
from unittest import TestCase
import orlo.util

__author__ = 'alforbes'


class TestUtil(TestCase):
    def test_str_to_bool_true(self):
        """
        Test some values for True
        """
        for v in ['true', 'TrUE', '1', 't', '99', 1, 99]:
            self.assertIs(orlo.util.str_to_bool(v), True)

    def test_str_to_bool_false(self):
        """
        Test some values for True
        """
        for v in ['false', 'FaLsE', 'f', '0', '-99', 0, -99]:
            self.assertIs(orlo.util.str_to_bool(v), False)



from __future__ import print_function
from test_route_base import OrloHttpTest
import json

__author__ = 'alforbes'


class GetPackagesTest(OrloHttpTest):
    """
    Test the HTTP GET contract
    """

    def _get_packages(self, package_id=None, filters=None, expected_status=200):
        """
        Perform a GET to /releases with optional filters
        """

        if package_id:
            path = '/packages/{}'.format(package_id)
        elif filters:
            path = '/packages?{}'.format('&'.join(filters))
        else:
            path = '/packages'

        results_response = self.client.get(
            path, content_type='application/json',
        )

        try:
            self.assertEqual(results_response.status_code, expected_status)
        except AssertionError as err:
            print(results_response.data)
            raise
        r_json = json.loads(results_response.data.decode('utf-8'))
        return r_json

    def test_packages(self):
        """
        Test that we get 400 and a message with no params
        """
        response = self.client.get('/packages')
        self.assert400(response)
        self.assertIn('message', response.json)

    def test_get_single_package(self):
        """
        Fetch a single release
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        p = self._get_packages(package_id=package_id)
        self.assertEqual(1, len(p['packages']))
        self.assertEqual(package_id, p['packages'][0]['id'])

    def test_get_package_multiple(self):
        """
        Fetch multiple packages
        """
        for _ in range(0, 3):
            self._create_finished_release()
        p = self._get_packages(filters=['name=test-package'])
        self.assertIsInstance(p['packages'], list)
        self.assertEqual(3, len(p['packages']))




import arrow
from flask import request, jsonify

from orlo import stats
from orlo import app
from orlo.exceptions import InvalidUsage
import orlo.queries as queries

__author__ = 'alforbes'

"""
#TODO The stats_ functions in this file are very similar, which means they should probably be
broken down a different way
"""


def build_stats_dict(field, value_list, platform=None, stime=None, ftime=None):
    """
    Build a dictionary of our stats

    :param field: The field we are build stats for, i.e. user, package, team or
        platform
    :param value_list: The list of values for the field
    :param platform: Filter by platform
    :param datetime ftime: Passed to count_releases
    :param datetime stime: Passed to count_releases
    :return:
    """

    app.logger.debug("Entered build_stats_dict")

    d_stats = {}
    count_releases_args = {
        'stime': stime,
        'ftime': ftime,
        'platform': platform,  # platform can also be the field in the loop below
    }

    for field_value in value_list:
        app.logger.debug("Getting stats for {} {}".format(field, field_value))

        count_releases_args[field] = field_value  # e.g. need to pass user=username

        c_total_successful = queries.count_releases(
                status='SUCCESSFUL', **count_releases_args).all()[0][0]
        c_total_failed = queries.count_releases(
                status='FAILED', **count_releases_args).all()[0][0]
        c_normal_successful = queries.count_releases(
                rollback=False, status='SUCCESSFUL', **count_releases_args).all()[0][0]
        c_normal_failed = queries.count_releases(
                rollback=False, status='FAILED', **count_releases_args).all()[0][0]
        c_rollback_successful = queries.count_releases(
                rollback=True, status='SUCCESSFUL', **count_releases_args).all()[0][0]
        c_rollback_failed = queries.count_releases(
                rollback=True, status='FAILED', **count_releases_args).all()[0][0]

        d_stats[field_value] = {
            'releases': {
                'normal': {
                    'successful': c_normal_successful,
                    'failed': c_normal_failed,
                },
                'rollback': {
                    'successful': c_rollback_successful,
                    'failed': c_rollback_failed,
                },
                'total': {
                    'successful': c_total_successful,
                    'failed': c_total_failed,
                },
            }
        }

    return d_stats


def build_all_stats_dict(stime=None, ftime=None):
    """
    Build a dictionary of our stats

    :param datetime ftime: Passed to count_releases
    :param datetime stime: Passed to count_releases

    :return:
    """

    app.logger.debug("Entered build_all_stats_dict")

    count_releases_args = {
        'stime': stime,
        'ftime': ftime,
    }

    app.logger.debug("Getting global stats")

    c_total_successful = queries.count_releases(
            status='SUCCESSFUL', **count_releases_args).all()[0][0]
    c_total_failed = queries.count_releases(
            status='FAILED', **count_releases_args).all()[0][0]
    c_normal_successful = queries.count_releases(
            rollback=False, status='SUCCESSFUL', **count_releases_args).all()[0][0]
    c_normal_failed = queries.count_releases(
            rollback=False, status='FAILED', **count_releases_args).all()[0][0]
    c_rollback_successful = queries.count_releases(
            rollback=True, status='SUCCESSFUL', **count_releases_args).all()[0][0]
    c_rollback_failed = queries.count_releases(
            rollback=True, status='FAILED', **count_releases_args).all()[0][0]

    d_stats = {
        'global': {
            'releases': {
                'normal': {
                    'successful': c_normal_successful,
                    'failed': c_normal_failed,
                },
                'rollback': {
                    'successful': c_rollback_successful,
                    'failed': c_rollback_failed,
                },
                'total': {
                    'successful': c_total_successful,
                    'failed': c_total_failed,
                },
            }
        }
    }

    return d_stats


@app.route('/stats')
def stats_():
    """
    Return dictionary of global stats

    :query string stime: The lower bound of the time period to filter on
    :query string ftime: The upper bound of the time period to filter on
    """
    s_stime = request.args.get('stime')
    s_ftime = request.args.get('ftime')

    stime, ftime = None, None
    try:
        if s_stime:
            stime = arrow.get(s_stime)
        if s_ftime:
            ftime = arrow.get(s_ftime)
    except RuntimeError:  # super-class to arrow's ParserError, which is not importable
        raise InvalidUsage("A badly formatted datetime string was given")

    app.logger.debug("Building all_stats dict")
    all_stats = build_all_stats_dict(stime=stime, ftime=ftime)

    return jsonify(all_stats)


@app.route('/stats/user')
@app.route('/stats/user/<username>')
def stats_user(username=None):
    """
    Return a dictionary of statistics for a username (optional), or all users

    :param string username: The username to provide u_stats for
    :query string stime: The lower bound of the time period to filter on
    :query string ftime: The upper bound of the time period to filter on

    stime and ftime both filter on the release start time.
    Note that standard orlo API filters can be used here as well, not just stime/ftime
    """
    s_stime = request.args.get('stime')
    s_ftime = request.args.get('ftime')

    stime, ftime = None, None
    try:
        if s_stime:
            stime = arrow.get(s_stime)
        if s_ftime:
            ftime = arrow.get(s_ftime)
    except RuntimeError:  # super-class to arrows ParserError, which is not importable
        raise InvalidUsage("A badly formatted datetime string was given")

    if username:
        user_list = [username]
    else:
        app.logger.debug("Fetching user list")
        user_result = queries.user_list().all()
        # Flatten query result
        user_list = [u[0] for u in user_result]

    app.logger.debug("Building stats dict")
    user_stats = build_stats_dict('user', user_list, stime=stime, ftime=ftime)

    return jsonify(user_stats)


@app.route('/stats/team')
@app.route('/stats/team/<team>')
def stats_team(team=None):
    """
    Return a dictionary of statistics for a team (optional), or all teams

    :param string team: The team name to provide t_stats for
    :query string stime: The lower bound of the time period to filter on
    :query string ftime: The upper bound of the time period to filter on

    stime and ftime both filter on the release start time.
    """
    s_stime = request.args.get('stime')
    s_ftime = request.args.get('ftime')

    stime, ftime = None, None
    try:
        if s_stime:
            stime = arrow.get(s_stime)
        if s_ftime:
            ftime = arrow.get(s_ftime)
    except RuntimeError:  # super-class to arrows ParserError, which is not importable
        raise InvalidUsage("A badly formatted datetime string was given")

    if team:
        team_list = [team]
    else:
        team_result = queries.team_list().all()
        # Flatten query result
        team_list = [u[0] for u in team_result]

    team_stats = build_stats_dict('team', team_list, stime=stime, ftime=ftime)

    return jsonify(team_stats)


@app.route('/stats/platform')
@app.route('/stats/platform/<platform>')
def stats_platform(platform=None):
    """
    Return a dictionary of statistics for a platform name (optional), or all platforms

    :param string platform: The platform name to provide stats for
    :query string stime: The lower bound of the time period to filter on
    :query string ftime: The upper bound of the time period to filter on

    stime and ftime both filter on the release start time.
    Note that standard orlo API filters can be used here as well, not just stime/ftime
    """
    s_stime = request.args.get('stime')
    s_ftime = request.args.get('ftime')

    try:
        stime, ftime = None, None
        if s_stime:
            stime = arrow.get(s_stime)
        if s_ftime:
            ftime = arrow.get(s_ftime)
    except RuntimeError:  # super-class to arrows ParserError, which is not importable
        raise InvalidUsage("A badly formatted datetime string was given")

    if platform:
        platform_list = [platform]
    else:
        platform_result = queries.platform_list().all()
        # Flatten query result
        platform_list = [u[0] for u in platform_result]

    platform_stats = build_stats_dict('platform', platform_list, stime=stime, ftime=ftime)

    return jsonify(platform_stats)


@app.route('/stats/package')
@app.route('/stats/package/<package>')
def stats_package(package=None):
    """
    Return a dictionary of statistics for a package name (optional), or all packages

    :param string package: The package name to provide p_stats for
    :query string stime: The lower bound of the time period to filter on
    :query string ftime: The upper bound of the time period to filter on

    stime and ftime both filter on the release start time.
    Note that standard orlo API filters can be used here as well, not just stime/ftime
    """
    s_stime = request.args.get('stime')
    s_ftime = request.args.get('ftime')

    stime, ftime = None, None
    try:
        if s_stime:
            stime = arrow.get(s_stime)
        if s_ftime:
            ftime = arrow.get(s_ftime)
    except RuntimeError:  # super-class to arrows ParserError, which is not importable
        raise InvalidUsage("A badly formatted datetime string was given")

    if package:
        package_list = [package]
    else:
        package_result = queries.package_list().all()
        # Flatten query result
        package_list = [u[0] for u in package_result]

    package_stats = build_stats_dict('package', package_list, stime=stime, ftime=ftime)

    return jsonify(package_stats)


@app.route('/stats/by_date/<subject>')
def stats_by_date(subject='release'):
    """
    Return stats by date

    :param subject: Release or Package (default: release)
    :query string unit: Unit to group by, i.e. year, month, week, day, hour
    :query boolean summarize_by_unit: Don't build hierarchy, just summarize by the unit
    :return:

    This endpoint also allows filtering on the same fields as GET /releases, e.g stime_gt. See
    that endpoint for documentation.
    """

    # Get the filters into an args directory, if they are set
    filters = dict((k, v) for k, v in request.args.items())
    unit = filters.pop('unit', 'month')

    if filters.pop('summarize_by_unit', False):
        summarize_by_unit = True
    else:
        summarize_by_unit = False

    # Returns releases and their time by rollback and status
    if subject == 'release':
        release_stats = stats.releases_by_time(unit, summarize_by_unit, **filters)
    elif subject == 'package':
        release_stats = stats.packages_by_time(unit, summarize_by_unit, **filters)
    else:
        raise InvalidUsage("subject must release or package, not '{}'".format())

    return jsonify(release_stats)





#!/usr/bin/env python

# from distutils.core import setup
from setuptools import setup
import multiprocessing  # nopep8
import os


__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))

VERSION = '0.3.1'

version_file = open(os.path.join(__location__, 'orlo', '_version.py'), 'w')
version_file.write("__version__ = '{}'".format(VERSION))
version_file.close()


setup(
    name='orlo',
    version=VERSION,
    description='Deployment data capture API',
    author='Alex Forbes',
    author_email='alforbes@ebay.com',
    license='GPL',
    long_description=open(os.path.join(__location__, 'README.md')).read(),
    url='https://github.com/eBayClassifiedsGroup/orlo',
    packages=[
        'orlo',
    ],
    include_package_data=True,
    install_requires=[
        'Flask',
        'Flask-Migrate',
        'Flask-SQLAlchemy',
        'Flask-HTTPAuth',
        'Flask-TokenAuth',
        'arrow',
        'gunicorn',
        'psycopg2',
        'pyldap',
        'pytz',
        'sphinxcontrib-httpdomain',
        'sqlalchemy-utils',
    ],
    tests_require=[
        'Flask-Testing',
        'orloclient>=0.1.1',
    ],
    test_suite='tests',
    # Creates a script in /usr/local/bin
    entry_points={
        'console_scripts': ['orlo=orlo.cli:main']
    }
)



from __future__ import print_function
import json
import subprocess
from threading import Timer
from orlo.config import config
from orlo.exceptions import OrloDeployError
from orlo import app

__author__ = 'alforbes'


class BaseDeploy(object):
    """
    A Deploy task

    Deploy tasks are simpler than releases, as they consist of just packages
    and versions. The backend deployer is responsible for creating Orlo Releases
    via the REST API.

    Integrations can either sub-class Deploy and override the start method and
    (optionally) the kill method, or use the pre-built shell and http
    integrations.
    """

    def __init__(self, release):
        """
        Perform a release
        """
        self.release = release
        self.server_url = config.get('main', 'base_url')

    def start(self):
        """
        Start the deployment
        """
        raise NotImplementedError("Please override the start method")

    def kill(self):
        """
        Kill a deployment in progress
        """
        raise NotImplementedError("Please override the kill method")


class HttpDeploy(BaseDeploy):
    """
    A http-based Deployment
    """

    def __init__(self, release):
        super(HttpDeploy, self).__init__(release)

    def start(self):
        pass

    def kill(self):
        raise NotImplementedError("No kill method for HTTP deploys")


class ShellDeploy(BaseDeploy):
    """
    Deployment by shell command

    Data is passed to the shell command given in 3 ways:

    * ORLO_URL, ORLO_RELEASE (the ID), and other Release attributes
      are set as environment variables (all prefixed by ORLO_)
    * The package and version sets are passed as arguments, e.g.
      package-name=1.0.0
    * The metadata dictionary is passed to stdin
    """

    def __init__(self, release):
        super(ShellDeploy, self).__init__(release)
        self.pid = None

    def start(self):
        """
        Start the deploy
        """
        args = [config.get('deploy_shell', 'command_path')]
        for p in self.release.packages:
            args.append("{}={}".format(p.name, p.version))
        app.logger.debug("Args: {}".format(str(args)))

        env = {
            'ORLO_URL': self.server_url,
            'ORLO_RELEASE': str(self.release.id)
        }
        for key, value in self.release.to_dict().items():
            my_key = "ORLO_" + key.upper()
            env[my_key] = json.dumps(value)

        app.logger.debug("Env: {}".format(json.dumps(env)))

        metadata = {}
        for m in self.release.metadata:
            metadata.update(m.to_dict())
        in_data = json.dumps(metadata)

        return self.run_command(
            args, env, in_data, timeout_sec=config.getint('deploy', 'timeout'))

    def kill(self):
        """
        Kill a deploy in progress
        """
        raise NotImplementedError

    @staticmethod
    def run_command(args, env, in_data, timeout_sec=3600):
        """
        Run a command in a separate thread

        Adapted from http://stackoverflow.com/questions/1191374

        :param env: Dict of environment variables
        :param in_data: String to pass to stdin
        :param list args: Arguments (i.e. full list including command,
        as you would pass to subprocess)
        :param timeout_sec: Timeout in seconds, 1 hour by default
        :return:
        """
        try:
            proc = subprocess.Popen(
                args,
                env=env,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
        except OSError as e:
            raise OrloDeployError(
                message="OSError starting process: {}".format(
                    e.strerror),
                payload={'arguments': args}
            )

        timer = Timer(timeout_sec, proc.kill)
        out = err = " "
        try:
            timer.start()
            out, err = proc.communicate(in_data.encode('utf-8'))
        finally:
            timer.cancel()
            app.logger.debug("Out:\n{}".format(out))
            app.logger.debug("Err:\n{}".format(err))

        if proc.returncode is not 0:
            raise OrloDeployError(
                message="Subprocess exited with code {}".format(
                    proc.returncode),
                payload={
                    'stdout': out,
                    'stderr': err,
                },
                status_code=500)
        else:
            app.logger.info(
                "Deploy completed successfully. Output:\n{}".format(out))
        app.logger.debug("end run")

        return {
            'stdout': out,
            'stderr': err,
        }



from __future__ import print_function
from test_base import OrloLiveTest, OrloTest, ConfigChange, ReleaseDbUtil
from orlo.deploy import BaseDeploy, HttpDeploy, ShellDeploy
from orlo.orm import db, Release, Package
from test_base import OrloLiveTest
from test_base import ReleaseDbUtil
import unittest

__author__ = 'alforbes'


class DeployTest(OrloLiveTest, ReleaseDbUtil):
    """
    Test the Deploy class
    """
    CLASS = BaseDeploy

    def setUp(self):
        super(DeployTest, self).setUp()
        rid = self._create_release()
        pid = self._create_package(rid)
        self.release = db.session.query(Release).first()

    def test_init(self):
        """
        Test that we can instantiate the class
        """
        o = self.CLASS(self.release)
        self.assertIsInstance(o, BaseDeploy)


class TestBaseDeploy(DeployTest):
    def test_not_implemented(self):
        """
        Base Deploy class should raise NotImplementedError on start
        """
        o = self.CLASS(self.release)
        with self.assertRaises(NotImplementedError):
            o.start()


class TestHttpDeploy(DeployTest):
    CLASS = HttpDeploy

    @unittest.skip("Not implemented")
    def test_start(self):
        """
        Test that start emits an http call
        """
        pass

    @unittest.skip("Not implemented")
    def test_kill(self):
        """
        Test that kill emits an http call
        """
        pass


class TestShellDeploy(DeployTest):
    CLASS = ShellDeploy

    def test_start(self):
        """
        Test that start emits a shell command
        """
        with ConfigChange('deploy', 'timeout', '3'), \
                ConfigChange('deploy_shell', 'command_path', '/bin/true'):
            deploy = ShellDeploy(self.release)

            # Override server_url, normally it is set by config:
            deploy.server_url = self.get_server_url()

            deploy.start()

    @unittest.skip("Not implemented")
    def test_kill(self):
        """
        Test that kill emits a shell command
        """
        pass

    @unittest.skip("Doesn't work on travis")
    def test_start_example_deployer(self):
        """
        Test the example deployer completes

        DOESN'T WORK ON TRAVIS, as /bin/env python gives the system python
        """
        with ConfigChange('deploy', 'timeout', '3'):
            deploy = ShellDeploy(self.release)

            # Override server_url, normally it is set by config:
            deploy.server_url = self.get_server_url()

            deploy.start()

    def test_output(self):
        """
        Test that we return the output of the deploy

        Not a good test, as it relies on the test-package being an argument,
        and simply echoing it back. This is the "spec", but this test could
        break if the arguments change.
        """
        with ConfigChange('deploy', 'timeout', '3'), \
                ConfigChange('deploy_shell', 'command_path', '/bin/echo'):
            deploy = ShellDeploy(self.release)

            # Override server_url, normally it is set by config:
            deploy.server_url = self.get_server_url()

            output = deploy.start()
            self.assertEqual(output['stdout'], b'test-package=1.2.3\n')




from __future__ import print_function
import os
from six.moves.configparser import RawConfigParser

__author__ = 'alforbes'

try:
    CONFIG_FILE = os.environ['ORLO_CONFIG']
except KeyError:
    CONFIG_FILE = '/etc/orlo/orlo.ini'

config = RawConfigParser()

config.add_section('main')
config.set('main', 'debug_mode', 'false')
config.set('main', 'propagate_exceptions', 'true')
config.set('main', 'time_format', '%Y-%m-%dT%H:%M:%SZ')
config.set('main', 'time_zone', 'UTC')
config.set('main', 'strict_slashes', 'false')
config.set('main', 'base_url', 'http://localhost:8080')

config.add_section('security')
config.set('security', 'enabled', 'false')
config.set('security', 'passwd_file', 'none')
config.set('security', 'secret_key', 'change_me')
# NOTE: orlo.__init__ checks that secret_key is not "change_me" when security
# is enabled. Do not change the default here without updating __init__ as well.
config.set('security', 'token_ttl', '3600')
config.set('security', 'ldap_server', 'localhost.localdomain')
config.set('security', 'ldap_port', '389')
config.set('security', 'user_base_dn', 'ou=people,ou=example,o=test')

config.add_section('db')
config.set('db', 'uri', 'sqlite://')
config.set('db', 'echo_queries', 'false')
config.set('db', 'pool_size', '50')

config.add_section('logging')
config.set('logging', 'level', 'info')
config.set('logging', 'file', 'disabled')
config.set('logging', 'format', '%(asctime)s [%(name)s] %(levelname)s %('
                                'module)s:%(funcName)s:%(lineno)d - %('
                                'message)s')

config.add_section('deploy')
config.set('deploy', 'timeout',
           '3600')  # How long to timeout external deployer calls

config.add_section('deploy_shell')
config.set('deploy_shell', 'command_path',
           os.path.dirname(os.path.abspath(__file__)) +
           '/../deployer.py')

config.read(CONFIG_FILE)



from __future__ import print_function
from test_orm import OrloDbTest
import arrow
import unittest

__author__ = 'alforbes'


class TestStats(OrloDbTest):
    ENDPOINT = '/stats'

    def setUp(self):
        super(OrloDbTest, self).setUp()
        for r in range(0, 3):
            self._create_finished_release()

    def test_endpoint_200(self):
        """
        Test self.ENDPOINT returns 200
        """
        response = self.client.get(self.ENDPOINT)
        self.assert200(response)

    def test_endpoint_returns_dict(self):
        """
        Test self.ENDPOINT returns a dictionary
        """
        response = self.client.get(self.ENDPOINT)
        self.assertIsInstance(response.json, dict)

    def test_result_includes_normals(self):
        """
        Test the result includes a 'normal' field
        """
        response = self.client.get(self.ENDPOINT)
        for k, v in response.json.items():
            self.assertIn('successful', v['releases']['normal'])
            self.assertIn('failed', v['releases']['normal'])

    def test_result_includes_rollbacks(self):
        """
        Test the result includes a 'rollback' field
        """
        response = self.client.get(self.ENDPOINT)
        for k, v in response.json.items():
            self.assertIn('successful', v['releases']['rollback'])
            self.assertIn('failed', v['releases']['rollback'])

    def test_result_includes_totals(self):
        """
        Test the result includes a 'successful' field
        """
        response = self.client.get(self.ENDPOINT)
        for k, v in response.json.items():
            self.assertIn('successful', v['releases']['total'])
            self.assertIn('failed', v['releases']['total'])

    def test_with_invalid_stime(self):
        """
        Test that an invalid start time is handled gracefully
        """
        response = self.client.get(self.ENDPOINT + '?stime=foo')
        self.assert400(response)
        self.assertIn('message', response.json)


class TestUserStats(TestStats):
    ENDPOINT = '/stats/user'

    def test_stats_user_200_with_user(self):
        """
        Test that /stats/user/username returns 200
        """
        response = self.client.get(self.ENDPOINT + '/testuser')
        self.assert200(response)

    def test_stats_user_returns_dict_with_user(self):
        """
        Test that /stats/user/username returns a dictionary
        """
        response = self.client.get(self.ENDPOINT + '/testuser')
        self.assertIsInstance(response.json, dict)


class TestTeamStats(TestStats):
    ENDPOINT = '/stats/team'

    def test_stats_team_200_with_team(self):
        """
        Test that /stats/team/team_name returns 200
        """
        response = self.client.get(self.ENDPOINT + '/test%20team')
        self.assert200(response)

    def test_stats_team_returns_dict_with_team(self):
        """
        Test that /stats/team/team_name returns a dictionary
        """
        response = self.client.get(self.ENDPOINT + '/test%20team')
        self.assertIsInstance(response.json, dict)


class TestPlatformStats(TestStats):
    ENDPOINT = '/stats/platform'

    def test_stats_platform_200_with_platform(self):
        """
        Test that /stats/platform/platform_name returns 200
        """
        response = self.client.get(self.ENDPOINT + '/test_platform')
        self.assert200(response)


class TestPackageStats(TestStats):
    ENDPOINT = '/stats/package'

    def test_stats_package_200_with_package(self):
        """
        Test that /stats/package/package_name returns 200
        """
        response = self.client.get(self.ENDPOINT + '/test-package')
        self.assert200(response)

    def test_stats_package_returns_dict_with_package(self):
        """
        Test that /stats/package/package_name returns a dictionary
        """
        response = self.client.get(self.ENDPOINT + '/test-package')
        self.assertIsInstance(response.json, dict)


class TestStatsByDateRelease(TestStats):
    """
    Testing the "by_date" urls
    """
    ENDPOINT = '/stats/by_date/release'

    def test_result_includes_normals(self):
        unittest.skip("Not suitable test for this endpoint")

    def test_result_includes_rollbacks(self):
        unittest.skip("Not suitable test for this endpoint")

    def test_result_includes_totals(self):
        unittest.skip("Not suitable test for this endpoint")

    def test_with_invalid_stime(self):
        # TODO the stats endpoints should be made consistent with the others by calling
        # apply_filters on the query parameters
        unittest.skip("Not suitable test for this endpoint")

    def test_stats_by_date_with_year(self):
        """
        Test /stats/by_date/year
        """
        year = arrow.utcnow().year
        response = self.client.get(self.ENDPOINT + '?stime_gt={}-01-01'.format(year))
        self.assert200(response)

    def test_stats_by_date_with_year_month(self):
        """
        Test /stats/by_date/year
        """
        year = arrow.utcnow().year
        month = arrow.utcnow().month
        response = self.client.get(self.ENDPOINT + '?stime_gt={}-{}-01'.format(year, month))
        self.assert200(response)

    def test_stats_by_date_with_unit_day(self):
        """
        Test /stats/by_date/year
        """
        response = self.client.get(self.ENDPOINT + '?unit=day')
        self.assert200(response)

    def test_stats_by_date_with_summarize_by_unit_day(self):
        """
        Test /stats/by_date/year
        """
        response = self.client.get(self.ENDPOINT + '?unit=day&summarize_by_unit=1')
        self.assert200(response)

    def test_stats_by_date_with_platform_filter(self):
        """
        Test /stats/by_date with a platform filter
        """
        year = str(arrow.utcnow().year)
        response = self.client.get(self.ENDPOINT + '?platform=test_platform')
        self.assert200(response)
        self.assertIn(year, response.json)

    def test_stats_by_date_with_platform_filter_negative(self):
        """
        Test /stats/by_date with a bad platform filter returns nothing
        """
        response = self.client.get(self.ENDPOINT + '?platform=bad_platform_foo')
        self.assert200(response)
        self.assertEqual({}, response.json)


class TestStatsByDatePackage(OrloDbTest):
    """
    Testing the "by_date" urls
    """
    ENDPOINT = '/stats/by_date/package'

    def setUp(self):
        super(OrloDbTest, self).setUp()
        for r in range(0, 3):
            self._create_finished_release()

    def test_endpoint_200(self):
        """
        Test self.ENDPOINT returns 200
        """
        response = self.client.get(self.ENDPOINT)
        self.assert200(response)

    def test_endpoint_returns_dict(self):
        """
        Test self.ENDPOINT returns a dictionary
        """
        response = self.client.get(self.ENDPOINT)
        self.assertIsInstance(response.json, dict)

    def test_package_name_in_dict(self):
        """
        Test the package name is in the returned json
        """
        response = self.client.get(self.ENDPOINT)
        year = str(arrow.utcnow().year)
        month = str(arrow.utcnow().month)
        self.assertIn('test-package', response.json[year][month])
